package com.cuit.basin.base.biz.mq.model;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.cuit.basin.base.biz.controller.model.vo.InitTaskParams;
import com.cuit.basin.base.biz.controller.model.vo.SuccessTaskParams;
import com.cuit.basin.base.biz.service.model.ModelOperationTaskService;
import com.cuit.basin.base.websocket.server.WebSocketServer;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.sriver.sriversys.mq.model
 * @Author: HouPJ
 * @Date: 2022/12/22 16:27
 * @Description: 模型相关消息消费者
 */
@Component
@Slf4j
public class ModelMQConsummer {

    @Resource
    private WebSocketServer webSocket;
    @Resource
    private ModelOperationTaskService taskService;

    /**
     * 监听模型实时日志处理消息
     * @param message
     * @param ack
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_LOG}, groupId = "model-log")
    public void listenModelLog(String message, Acknowledgment ack) {
        log.info("接收到主题为{}的消息，size:{}",KafkaTopic.TOPIC_MODEL_LOG,message.length());
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        //利用websocket通知前端刷新页面
        log.info("消息为" + message + "开始利用websocket通知前端.....");
        try{
            webSocket.sendGroupInfo(message);
        }catch (Exception e) {
            log.error("websocket 消息发送失败！");
        }
        // 手动提交偏移量
        try{
            ack.acknowledge();
        }catch (Exception e){
            log.error("消费模型启动消息异常，errMsg:{}", e.getMessage(),e);
            throw new ErrorMessage("model-exception:[MODEL_LOG]消息消费失败....");
        }
    }


    /**
     * 监听模型初始化消息
     * @param message
     * @param ack
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_INIT}, groupId = "model-init",concurrency = "3")
    public void listenModelInit(String message, Acknowledgment ack) {
        log.info("接收到主题为{}的消息，内容为:{}",KafkaTopic.TOPIC_MODEL_INIT,message);
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        // 调用模型初始化时服务
        taskService.initModelTask(JSONUtil.toBean(message, InitTaskParams.class));
        // 手动提交偏移量
        try{
            ack.acknowledge();
        }catch (Exception e){
            log.error("消费模型启动消息异常，errMsg:{}", e.getMessage(),e);
            throw new ErrorMessage("model-exception:[MODEL_EXCEPTION]消息消费失败....");
        }
    }


    /**
     * 监听模型进度更新结果
     * @param message
     * @param ack
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_PROCESS}, groupId = "model-progress",concurrency = "3")
    public void listenModelProcess(String message, Acknowledgment ack) {
        log.info("接收到主题为{}的消息，内容为:{}",KafkaTopic.TOPIC_MODEL_PROCESS,message);
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        // 转换message
        SuccessTaskParams successTaskParams = JSONUtil.toBean(message, SuccessTaskParams.class);
        // 调用模型成功结束服务
        taskService.modelTaskRefreshHandler(successTaskParams);
        // 手动提交偏移量
        try{
            ack.acknowledge();
        }catch (Exception e){
            log.error("消费模型启动消息异常，errMsg:{}", e.getMessage(),e);
            throw new ErrorMessage("model-exception:[MODEL_PROCESS]消息消费失败....");
        }
    }

    /**
     * 监听模型运行成功消息
     * @param message
     * @param ack
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_SUCCESS}, groupId = "model-success",concurrency = "3")
    public void listenModelSuccess(String message, Acknowledgment ack) {
        log.info("接收到主题为{}的消息，内容为:{}",KafkaTopic.TOPIC_MODEL_SUCCESS,message);
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        // 转换message
        SuccessTaskParams successTaskParams = JSONUtil.toBean(message, SuccessTaskParams.class);
        // 调用模型成功结束服务
        taskService.modelTaskRefreshHandler(successTaskParams);
        // 手动提交偏移量
        try{
            ack.acknowledge();
        }catch (Exception e){
            log.error("消费模型启动消息异常，errMsg:{}", e.getMessage(),e);
            throw new ErrorMessage("model-exception:[MODEL_SUCCESS]消息消费失败....");
        }
    }

    /**
     * 监听模型异常消息
     * @param message
     * @param ack
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_EXCEPTION}, groupId = "model-exception",concurrency = "3")
    public void listenModelException(String message, Acknowledgment ack) {
        log.info("接收到主题为{}的消息，内容为:{}",KafkaTopic.TOPIC_MODEL_EXCEPTION,message);
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        // 调用模型发生异常时服务
        taskService.modelTaskExceptionHandler(message);
        // 手动提交偏移量
        try{
            ack.acknowledge();
        }catch (Exception e){
            log.error("消费模型启动消息异常，errMsg:{}", e.getMessage(),e);
            throw new ErrorMessage("model-exception:[MODEL_EXCEPTION]消息消费失败....");
        }
    }


}
