package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.entity.HydroStationInfo;
import com.cuit.basin.base.biz.entity.MeteStationData;
import com.cuit.basin.base.biz.entity.MeteStationForcastingData;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.ret.PageVo;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 实时监测模块基础数据服务接口
 */
@Service
public interface RTMonitoringDetailsService {
    /**
     * 按固定时段查询历史水位流量
     *
     * @param station_id_c 站号
     * @param period       固定时段
     * @return HydroHistoryView
     */
    HydroHistoryView queryHistoryHydroDataByPeriod(String station_id_c, String period);

    /**
     * 按自定义时间范围查询历史水位流量
     *
     * @param station_id_c 站号
     * @param starttime    开始时间
     * @param endtime      结束时间
     * @return HydroHistoryView
     */
    HydroHistoryView queryHistoryHydroDataByTimeRange(String station_id_c, LocalDateTime starttime, LocalDateTime endtime);

    /**
     * 按固定时段查询历史断面水位
     *
     * @param stationidc 站号
     * @param period     固定时段
     * @return SectionLevelView
     */
    SectionLevelView queryHistorySectionDataByPeriod(String stationidc, String period);

    /**
     * 按时间范围查询历史断面水位
     *
     * @param stationidc 站号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return SectionLevelView
     */
    SectionLevelView queryHistorySectionDataByTimeRange(String stationidc, LocalDateTime starttime, LocalDateTime endtime);

    /**
     * 按固定时段查询历史同期对比（近5年）
     *
     * @param stationidc 站点编号
     * @param period     时段
     * @return DataCompareView
     */
    DataCompareView queryHistoryComparisonByPeriod(String stationidc, String period);

    /**
     * 按时间范围查询历史同期对比（近5年）
     *
     * @param stationidc 站点编号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return DataCompareView
     */
    DataCompareView queryHistoryComparisonByTimeRange(String stationidc, LocalDateTime starttime, LocalDateTime endtime);

    /**
     * 查询历史同期对比的所有年份
     *
     * @return List<Integer>
     */
    List<Integer> queryHistoryComparisonYears();

    /**
     * 按固定时段查询历史同期对比-更多年份
     *
     * @param stationidc 站号
     * @param period     固定时段
     * @param year       年份字符串列表
     * @return DataCompareView
     */
    DataCompareView queryHistoryComparisonByPeriodMore(String stationidc, String period, String year);

    /**
     * 按时间范围查询历史同期对比-更多年份
     *
     * @param stationidc 站号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @param year       年份字符串列表
     * @return DataCompareView
     */
    DataCompareView queryHistoryComparisonByTimeRangeMore(String stationidc, LocalDateTime starttime, LocalDateTime endtime, String year);

    /**
     * 按固定时段查询多站对比
     *
     * @param stationIdc 站点编号
     * @param period     时段
     * @return DataCompareView
     */
    DataCompareView queryStationComparisonByPeriod(String stationIdc, String period);

    /**
     * 按时间范围查询多站对比
     *
     * @param stationIdc 站点编号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return DataCompareView
     */
    DataCompareView queryStationComparisonByTimeRange(String stationIdc, LocalDateTime starttime, LocalDateTime endtime);

    /**
     * 按年查询历史值对比
     *
     * @param stationidc     站点编号
     * @param currentyear    当前年
     * @param comparisonyear 对比年
     * @param querytype      查询类型
     * @return DataCompareView
     */
    DataHisValueCompareView queryHistoryComparisonByYear(String stationidc, Integer currentyear, Integer comparisonyear, String querytype);

    /**
     * 按月查询历史值对比
     *
     * @param stationidc      站点编号
     * @param currentmonth    当前年月
     * @param comparisonmonth 对比年月
     * @param querytype       查询类型
     * @return DataCompareView
     */
    DataHisValueCompareView queryHistoryComparisonByMonth(String stationidc, String currentmonth, String comparisonmonth, String querytype);

    /**
     * 按日查询历史值对比
     *
     * @param stationidc    站点编号
     * @param currentday    当前年月日
     * @param comparisonday 对比年月日
     * @param querytype     查询类型
     * @return DataCompareView
     */
    DataHisValueCompareView queryHistoryComparisonByDay(String stationidc, LocalDate currentday, LocalDate comparisonday, String querytype);

    /**
     * 查询相似年对比
     *
     * @param stationIdc 站点编号
     * @return DataCompareView
     */
    DataSimilarView queryHistorySimilarityComparison(String stationIdc);

    /**
     * 按条件查询报汛特征值
     *
     * @param stationIdc 站号
     * @param startTime  开始时间
     * @param endTime    结束时间
     * @param query      查询类
     * @return HydroMonthCharacterValueView
     */
    HydroMonthCharacterValueView queryMonthCharacteristicValue(String stationIdc, String startTime, String endTime, BasePageQuery query);

    /**
     * 按条件查询报汛特征值的历史对比
     *
     * @param stationIdc 站号
     * @param startTime  开始时间
     * @param endTime    结束时间
     * @param dataType   数据类型
     * @param query      查询类
     * @return HydroMonthCharacterValueView
     */
    HydroMonthCharacterValueCompareView queryMonthCharacteristicValueCompare(String stationIdc, String startTime, String endTime, String dataType, BasePageQuery query);

    /**
     * 多条件查询特征值排序信息
     *
     * @param stationIdc 站号
     * @param datatype   数据类型
     * @param period     对比周期
     * @param startTime  开始周期
     * @param endTime    结束周期
     * @param orderType  排序方式
     * @param querytype  查询类型（确认/距平）
     * @return CharacteristicValueOrderView
     */
    CharacteristicValueOrderView queryCharacteristicValueOrder(String stationIdc, String datatype, String period, String startTime, String endTime, String orderType, String querytype);

//    /**
//     * 按时间范围查询径流组成信息
//     *
//     * @param stationIdc 站号
//     * @param stTime     开始时间
//     * @param endTime    结束时间
//     * @return HydroComposition
//     */
//    HydroCompositionView queryHydroCompositionByTimeRange(String stationIdc, LocalDateTime stTime, LocalDateTime endTime);

    /**
     * 按照时间查询水位流量关系信息
     *
     * @param stationIdc 站号
     * @param year       统计年
     * @param month      统计月
     * @return HydroStationDataView
     */
    HydroStationDataView queryHydroStationDataList(String stationIdc, Integer year, Integer month);

    /**
     * 按照时间范围查询雨量过程信息
     *
     * @param stationIdc 站号
     * @param stTime     开始时间
     * @param endTime    结束时间
     * @return RainfallProcessView
     */
    RainfallProcessView queryRainfallProcessListByTimeRange(String stationIdc, LocalDateTime stTime, LocalDateTime endTime);

    /**
     * 按照时间范围和流域查询流域站点面雨量信息
     *
     * @param stTime   开始时间
     * @param endTime  结束时间
     * @param bsidList 流域id字符串列表
     * @return WatershedStationArealRainfallView
     */
    WatershedStationArealRainfallView queryWatershedStationArealRainfallList(LocalDateTime stTime, LocalDateTime endTime, String bsidList);

    /**
     * 按照时间和流域查询流域站点实况信息
     *
     * @param time 时间
     * @param bsid 流域id
     * @return PageVo<HydroStationData>
     */
    PageVo<MeteStationData> queryMeteStationDataByTimeAndBsid(LocalDateTime time, String bsid);

    /**
     * 按照条件查询流域站点预报信息
     *
     * @return PageVo<HydroStationData>
     */
    PageVo<MeteStationForcastingData> queryMeteStationForecastDataByTimeAndBsid(LocalDateTime dataTime, Integer validTime, String bsid, String type);

    /**
     * 查询最新流域气象预报信息
     *
     * @return WatershedMeteForecastView
     */
    WatershedMeteForecastView queryWatershedMeteForecastList(LocalDateTime dataTime, Integer validTime);

    /**
     * 查询测站信息
     *
     * @param stationIdc 站号
     * @return HydroStationInfo
     */
    HydroStationInfo queryHydroStationInfo(String stationIdc);

    /**
     * 按固定时段查询径流组成信息
     *
     * @return HydroComposition
     */
    HydroCompositionView queryHydroCompositionByPeriod(String stationIdc, Integer mid, LocalDateTime dateTime, Integer validTime);

    /**
     * 按照固定时段查询雨量过程信息
     *
     * @param stationIdc 站点编号
     * @param period     时段
     * @return RainfallProcessView
     */
    RainfallProcessView queryRainfallProcessListByPeriod(String stationIdc, String period);

    /**
     * 按照固定时段和流域查询流域站点面雨量信息
     *
     * @param period   时段
     * @param bsidList 流域id字符串列表
     * @return WatershedStationArealRainfallView
     */
    WatershedStationArealRainfallView queryWatershedStationArealRainfallListByPeriod(String period, String bsidList);
}
