package com.cuit.basin.base.biz.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.basin.base.biz.dao.HydroEventMapper;
import com.cuit.basin.base.biz.dao.HydroHistoryCaseMapper;
import com.cuit.basin.base.biz.dao.HydroStationDataMapper;
import com.cuit.basin.base.biz.dao.HydroStationInfoMapper;
import com.cuit.basin.base.biz.entity.HydroEvent;
import com.cuit.basin.base.biz.entity.HydroHistoryCase;
import com.cuit.basin.base.biz.entity.HydroStationData;
import com.cuit.basin.base.biz.entity.HydroStationInfo;
import com.cuit.basin.base.biz.service.HistoricalCasesService;
import com.cuit.basin.base.biz.vo.HistoricalCasesDetailView;
import com.cuit.basin.base.biz.vo.HistoricalCasesView;
import com.cuit.basin.base.biz.vo.HydrologicalEventView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.constant.TimeLast;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.PageVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 历史个例板块Service
 *
 * @author Xiao Yinrui
 */
@Slf4j
@Service
public class HistoricalCasesServiceImpl implements HistoricalCasesService {

    DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    @Resource
    private HydroEventMapper hydroEventMapper;
    @Resource
    private HydroHistoryCaseMapper hydroHistoryCaseMapper;
    @Resource
    private HydroStationDataMapper hydroStationDataMapper;
    @Resource
    private HydroStationInfoMapper hydroStationInfoMapper;

    /**
     * 多条件查询水文事件列表信息（时间范围）
     *
     * @param type    类型
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param keyword 关键词
     * @param query   查询类
     * @return HydrologicalEventView
     */
    @Override
    public HydrologicalEventView getHydrologicalEventListByTime(String type, String stTime, String endTime, String keyword, BasePageQuery query) {
        Page<HydroEvent> page = new Page<>(query.getCurrent(), query.getSize());
        hydroEventMapper.selectPage(page, new LambdaQueryWrapper<HydroEvent>()
                .eq(!ObjectUtils.isEmpty(type), HydroEvent::getType, type)
                .like(!ObjectUtils.isEmpty(keyword), HydroEvent::getName, keyword)
                .like(!ObjectUtils.isEmpty(keyword), HydroEvent::getDes, keyword)
                .between(!ObjectUtils.isEmpty(stTime) && !ObjectUtils.isEmpty(endTime),
                        HydroEvent::getTm, stTime, endTime));
        String stcds = page.getRecords().stream().map(HydroEvent::getStcd).distinct().collect(Collectors.joining(","));
        List<HydroStationInfo> stationInfoList = hydroStationInfoMapper.selectList(new LambdaQueryWrapper<HydroStationInfo>()
                .apply("FIND_IN_SET(stcd, '" + stcds + "')"));
        return HydrologicalEventView.builder().events(new PageVo<>(page)).stationInfoList(stationInfoList).build();
    }

    /**
     * 多条件查询水文事件列表信息（固定时段）
     *
     * @param type    类型
     * @param period  时间段
     * @param keyword 关键词
     * @param query   查询类
     * @return HydrologicalEventView
     */
    @Override
    public HydrologicalEventView getHydrologicalEventListByPeriod(String type, String period, String keyword, BasePageQuery query) {
        // 固定时段转时间范围
        String startTime, endTime;
        if (ObjectUtils.isEmpty(period)) {
            startTime = endTime = null;
        } else {
            startTime = convertPeriodToTimeRange(period).format(df);
            endTime = LocalDateTime.now().format(df);
        }
        return getHydrologicalEventListByTime(type, startTime, endTime, keyword, query);
    }

    /**
     * 新增历史个例（从水文事件新增）
     *
     * @return boolean
     */
    @Override
    public boolean addHistoryCaseFromEvent() {
        return false;
    }

    /**
     * 删除水文事件
     *
     * @param eids 主键id列表
     * @return boolean
     */
    @Override
    public boolean deleteHydrologicalEvent(String eids) {
        return hydroEventMapper.deleteBatchIds(Arrays.stream(eids.replace("，", ",")
                .split(",")).collect(Collectors.toList())) > 0;
    }

    /**
     * 多条件查询历史个例信息（时间范围）
     *
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param keyword 关键词
     * @param query   查询类
     * @return HistoricalCasesView
     */
    @Override
    public HistoricalCasesView getHistoricalCasesByTime(String stTime, String endTime, String keyword, BasePageQuery query) {
        Page<HydroHistoryCase> page = new Page<>(query.getCurrent(), query.getSize());
        hydroHistoryCaseMapper.selectPage(page, new LambdaQueryWrapper<HydroHistoryCase>()
                .like(!ObjectUtils.isEmpty(keyword), HydroHistoryCase::getName, keyword)
                .like(!ObjectUtils.isEmpty(keyword), HydroHistoryCase::getDes, keyword)
                .ge(!ObjectUtils.isEmpty(stTime), HydroHistoryCase::getStm, stTime)
                .le(!ObjectUtils.isEmpty(endTime), HydroHistoryCase::getEtm, endTime));
        return HistoricalCasesView.builder().historyCases(new PageVo<>(page)).build();
    }

    /**
     * 多条件查询历史个例信息（固定时段）
     *
     * @param period  时间段
     * @param keyword 关键词
     * @param query   查询类
     * @return HistoricalCasesView
     */
    @Override
    public HistoricalCasesView getHistoricalCasesByPeriod(String period, String keyword, BasePageQuery query) {
        // 固定时段转时间范围
        String startTime, endTime;
        if (ObjectUtils.isEmpty(period)) {
            startTime = endTime = null;
        } else {
            startTime = convertPeriodToTimeRange(period).format(df);
            endTime = LocalDateTime.now().format(df);
        }
        return getHistoricalCasesByTime(startTime, endTime, keyword, query);
    }

    /**
     * 查询历史个例详情信息
     *
     * @param cid   历史个例id
     * @param query 查询类
     * @return HistoricalCasesDetailView
     */
    @Override
    public HistoricalCasesDetailView getHistoricalCasesDetail(Integer cid, BasePageQuery query) {
        HydroHistoryCase historyCase = hydroHistoryCaseMapper.selectById(cid);
        PageVo<HydroStationData> hydroStationData = getHydroStationData(historyCase.getStm(), historyCase.getEtm(), query);
        return HistoricalCasesDetailView.builder().historyCase(historyCase).stationData(hydroStationData).build();
    }

    /**
     * 新增历史个例信息
     *
     * @param name    历史个例名
     * @param desc    历史个例描述
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @return boolean
     */
    @Override
    public boolean addHistoricalCase(String name, String desc, LocalDateTime stTime, LocalDateTime endTime) {

        List<HydroStationData> stationDataList = getHydroStationData(stTime, endTime, BasePageQuery.builder().current(1).size(-1).build()).getRecords();
        // 计算最大、最小、平均的流量/水位
        HydroStationData maxZt = stationDataList.stream()
                .max(Comparator.comparing(HydroStationData::getZ)).orElse(HydroStationData.builder().z(-999.0).build());
        HydroStationData maxQt = stationDataList.stream()
                .max(Comparator.comparing(HydroStationData::getQ)).orElse(HydroStationData.builder().q(-999.0).build());
        Double minZ = stationDataList.stream()
                .min(Comparator.comparing(HydroStationData::getZ)).orElse(HydroStationData.builder().z(-999.0).build()).getZ();
        Double minQ = stationDataList.stream()
                .min(Comparator.comparing(HydroStationData::getQ)).orElse(HydroStationData.builder().q(-999.0).build()).getQ();
        double avgZ = stationDataList.stream()
                .mapToDouble(HydroStationData::getZ).average().orElse(-999);
        double avgQ = stationDataList.stream()
                .mapToDouble(HydroStationData::getQ).average().orElse(-999);

        return hydroHistoryCaseMapper.insert(HydroHistoryCase.builder()
                .name(name).des(desc).stm(stTime).etm(endTime)
                .mxz(maxZt.getZ()).mxztm(maxZt.getDt())
                .mxq(maxQt.getQ()).mxqtm(maxQt.getDt())
                .avgz(avgZ).avgq(avgQ).mnz(minZ).mnq(minQ)
                .build()) > 0;
    }


    /**
     * 按照时间范围查询水文站点数据
     *
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param query   查询类
     * @return PageVo<HydroStationData>
     */
    @Override
    public PageVo<HydroStationData> getHydroStationData(LocalDateTime stTime, LocalDateTime endTime, BasePageQuery query) {
        Page<HydroStationData> page = new Page<>(query.getCurrent(), query.getSize());
        hydroStationDataMapper.selectPage(page, new LambdaQueryWrapper<HydroStationData>()
                .between(HydroStationData::getDt, df.format(stTime), df.format(endTime)));
        return new PageVo<>(page);
    }

    /**
     * 批量删除历史个例
     *
     * @param cids 历史个例id列表
     * @return boolean
     */
    @Override
    public boolean deleteHistoricalCases(String cids) {
        return hydroHistoryCaseMapper.deleteBatchIds(Arrays.stream(cids.replace("，", ",")
                .split(",")).collect(Collectors.toList())) > 0;
    }

    /**
     * 固定时段转时间范围
     *
     * @param period 固定时段
     * @return startTime 时间范围的开始时间
     */
    private LocalDateTime convertPeriodToTimeRange(String period) {
        // 按照条件推算 开始时间
        if (period.equals(TimeLast.LAST_12_HOUR.getName())) {
            return DateUtil.offsetHour(new Date(), -12).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_24_HOUR.getName())) {
            return DateUtil.offsetHour(new Date(), -24).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_3_DAY.getName())) {
            return DateUtil.offsetDay(new Date(), -3).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_1_WEEK.getName())) {
            return DateUtil.offsetWeek(new Date(), -1).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_1_MONTH.getName())) {
            return DateUtil.offsetMonth(new Date(), -1).toLocalDateTime();
        } else {
            throw new ParamsException("指定固定时段错误！可用固定时段：" + Arrays.stream(TimeLast.values())
                    .map(TimeLast::getName).collect(Collectors.joining("，")));
        }
    }
}
