package com.cuit.basin.base.biz.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.basin.base.biz.dao.*;
import com.cuit.basin.base.biz.entity.*;
import com.cuit.basin.base.biz.pojo.*;
import com.cuit.basin.base.biz.service.RTMonitoringDetailsService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.constant.TimeLast;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.PageVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
 * 实时监测基础数据接口实现类
 */
@Slf4j
@Service
public class RTMonitoringDetailsServiceImpl implements RTMonitoringDetailsService {

    private final DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    @Resource
    private HydroStationHistoryMapper hydroStationHistoryMapper;
    @Resource
    private HydroStationDataMapper hydroStationDataMapper;
    @Resource
    private HydroStationInfoMapper hydroStationInfoMapper;
    @Resource
    private SectionDataMapper sectionDataMapper;
    @Resource
    private HydroCompositionMapper hydroCompositionMapper;
    @Resource
    private MeteRainfallProcessMapper meteRainfallProcessMapper;
    @Resource
    private WatershedStationArealRainfallMapper watershedStationArealRainfallMapper;
    @Resource
    private HydroYearCharacteristicValueMapper hydroYearCharacteristicValueMapper;
    @Resource
    private HydroDayCharacteristicValueMapper hydroDayCharacteristicValueMapper;
    @Resource
    private HydroMonthCharacterLevelValueMapper hydroMonthCharacterLevelValueMapper;
    @Resource
    private HydroMonthCharacterFlowValueMapper hydroMonthCharacterFlowValueMapper;
    @Resource
    private HydroHistorySimilarValueMapper hydroHistorySimilarValueMapper;
    @Resource
    private HydroMonthCharacterFlowHistoryValueMapper hydroMonthCharacterFlowHistoryValueMapper;
    @Resource
    private HydroMonthCharacterLevelHistoryValueMapper hydroMonthCharacterLevelHistoryValueMapper;
    @Resource
    private WatershedMeteForecastMapper watershedMeteForecastMapper;

    /**
     * 按固定时段查询历史水位流量
     *
     * @param stationIdC 站号
     * @param period     固定时段
     * @return HydroHistoryView
     */
    @Override
    public HydroHistoryView queryHistoryHydroDataByPeriod(String stationIdC, String period) {
        // 固定时段转时间范围
        LocalDateTime startTime = convertPeriodToTimeRange(period);
        return queryHistoryHydroDataByTimeRange(stationIdC, startTime, LocalDateTime.now());
    }

    /**
     * 按自定义时间范围查询历史水位流量
     *
     * @param stationIdC 站号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return HydroHistoryView
     */
    @Override
    public HydroHistoryView queryHistoryHydroDataByTimeRange(String stationIdC, LocalDateTime starttime, LocalDateTime endtime) {

        HydroStationHistory stationHistory = hydroStationHistoryMapper.selectOne(new LambdaQueryWrapper<HydroStationHistory>()
                .eq(HydroStationHistory::getStcd, stationIdC));
        HydroStationInfo stationInfo = hydroStationInfoMapper.selectOne(new LambdaQueryWrapper<HydroStationInfo>()
                .eq(HydroStationInfo::getStcd, stationIdC));
        List<HydroStationData> dataList = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                .eq(HydroStationData::getStcd, stationIdC).between(HydroStationData::getDt, df.format(starttime), df.format(endtime)));
        return HydroHistoryView.builder()
                .hydroStationHistory(stationHistory)
                .stationInfo(stationInfo)
                .data(dataList)
                .build();
    }

    /**
     * 按固定时段查询历史断面水位
     *
     * @param stationIdC 站号
     * @param period     固定时段
     * @return SectionLevelView
     */
    @Override
    public SectionLevelView queryHistorySectionDataByPeriod(String stationIdC, String period) {
        // 固定时段转时间范围
        LocalDateTime startTime = convertPeriodToTimeRange(period);
        return queryHistorySectionDataByTimeRange(stationIdC, startTime, LocalDateTime.now());
    }

    /**
     * 按时间范围查询历史断面水位
     *
     * @param stationIdC 站号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return SectionLevelView
     */
    @Override
    public SectionLevelView queryHistorySectionDataByTimeRange(String stationIdC, LocalDateTime starttime, LocalDateTime endtime) {

        HydroStationHistory history = hydroStationHistoryMapper.selectOne(new LambdaQueryWrapper<HydroStationHistory>()
                .eq(HydroStationHistory::getStcd, stationIdC));
        HydroStationInfo stationInfo = hydroStationInfoMapper.selectOne(new LambdaQueryWrapper<HydroStationInfo>()
                .eq(HydroStationInfo::getStcd, stationIdC));
        List<SectionData> sectionDataList = sectionDataMapper.selectList(new LambdaQueryWrapper<SectionData>()
                .eq(SectionData::getStcd, stationIdC));
        List<HydroStationData> dataList = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                .eq(HydroStationData::getStcd, stationIdC).between(HydroStationData::getDt, df.format(starttime), df.format(endtime)));
        return SectionLevelView.builder()
                .hydroStationHistory(history)
                .stationInfo(stationInfo)
                .sectionDataList(sectionDataList)
                .hydroStationDataList(dataList)
                .build();
    }

    /**
     * 按固定时段查询历史同期对比（近5年）
     *
     * @param stationIdC 站点编号
     * @param period     时段
     * @return DataCompareView
     */
    @Override
    public DataCompareView queryHistoryComparisonByPeriod(String stationIdC, String period) {
        // 固定时段转时间范围
        LocalDateTime startTime = convertPeriodToTimeRange(period);
        return queryHistoryComparisonByTimeRange(stationIdC, startTime, LocalDateTime.now());
    }

    /**
     * 按时间范围查询历史同期对比（近5年）
     *
     * @param stationIdC 站点编号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return DataCompareView
     */
    @Override
    public DataCompareView queryHistoryComparisonByTimeRange(String stationIdC, LocalDateTime starttime, LocalDateTime endtime) {
        // 查询最近5年组成的List（当前年除外），并转换为字符串列表（英文逗号分隔）
        String historyYearList = hydroYearCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroYearCharacteristicValue>()
                        .select(HydroYearCharacteristicValue::getYr)
                        .eq(HydroYearCharacteristicValue::getStcd, stationIdC)
                        .groupBy(HydroYearCharacteristicValue::getYr)
                        .orderByDesc(HydroYearCharacteristicValue::getYr)
                        .last("LIMIT 5")).stream()
                .map(HydroYearCharacteristicValue::getYr).map(String::valueOf)
                .collect(Collectors.joining(","));
        return queryHistoryComparisonByTimeRangeMore(stationIdC, starttime, endtime, historyYearList);
    }

    /**
     * 查询历史同期对比的所有年份
     *
     * @return List<Integer>
     */
    @Override
    public List<Integer> queryHistoryComparisonYears() {
        // 总年数
        return hydroYearCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroYearCharacteristicValue>()
                        .select(HydroYearCharacteristicValue::getYr)
                        .groupBy(HydroYearCharacteristicValue::getYr)
                        .orderByDesc(HydroYearCharacteristicValue::getYr)).stream()
                .map(HydroYearCharacteristicValue::getYr).collect(toList());
    }

    /**
     * 按固定时段查询历史同期对比
     *
     * @param stationidc 站号
     * @param period     固定时段
     * @param year       年份字符串列表
     * @return DataCompareView
     */
    @Override
    public DataCompareView queryHistoryComparisonByPeriodMore(String stationidc, String period, String year) {
        // 固定时段转时间范围
        LocalDateTime startTime = convertPeriodToTimeRange(period);
        return queryHistoryComparisonByTimeRangeMore(stationidc, startTime, LocalDateTime.now(), year);
    }

    /**
     * 按时间范围查询历史同期对比
     *
     * @param stationIdC 站号
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @param year       年份字符串列表（英文逗号隔开）
     * @return DataCompareView
     */
    @Override
    public DataCompareView queryHistoryComparisonByTimeRangeMore(String stationIdC, LocalDateTime starttime, LocalDateTime endtime, String year) {

        ArrayList<DataCompare> dataCompareList = new ArrayList<>();
        ArrayList<HydroStationData> hydroStationDataList = new ArrayList<>();
        // 1、将年份字符串转换为List
        List<Integer> historyYearList = Arrays.stream(year.replaceAll("，", ",").split(","))
                .map(Integer::new).collect(toList());
        // 2、判断并处理开始时间跨年问题
        starttime = handleOverYear(starttime, endtime);
        // 3、依次循环查询历年的数据
        for (Integer eachYear : historyYearList) {
            // 向前推算 历史开始时间 和 历史结束时间
            LocalDateTime offsetStartTime = LocalDateTimeUtil.offset(starttime, -1L * (starttime.getYear() - eachYear), ChronoUnit.YEARS);
            LocalDateTime offsetEndTime = LocalDateTimeUtil.offset(endtime, -1L * (endtime.getYear() - eachYear), ChronoUnit.YEARS);
            List<HydroStationData> dataList = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                    .eq(HydroStationData::getStcd, stationIdC)
                    .between(HydroStationData::getDt, df.format(offsetStartTime), df.format(offsetEndTime)));
            List<HydroStationDataValue> dataValueList = new ArrayList<>();
            for (HydroStationData each : dataList) {
                dataValueList.add(HydroStationDataValue.builder().dt(df.format(each.getDt())).z(each.getZ()).q(each.getQ()).build());
            }
            dataCompareList.add(DataCompare.builder()
                    .time(eachYear)
                    .stcd(stationIdC)
                    .data(dataValueList).build());
            // 每一年的时间 顺序
            dataList.sort(Comparator.comparing(HydroStationData::getDt));
            hydroStationDataList.addAll(dataList);
        }
        // 4、将数据按照年份逆序排列
        dataCompareList.sort(Comparator.comparing(DataCompare::getTime, Comparator.reverseOrder()));
        // 按照年份整体 逆序（但每一年的时间 顺序）
        hydroStationDataList.sort(Comparator.comparing(HydroStationData::getDt, (o1, o2) -> o2.getYear() - o1.getYear()));
        return DataCompareView.builder().
                dataCompares(dataCompareList).
                dataCompareList(hydroStationDataList).build();
    }

    /**
     * 按固定时段查询多站对比
     *
     * @param stationIdc 站点编号（多个,隔开）
     * @param period     时段
     * @return DataCompareView
     */
    @Override
    public DataCompareView queryStationComparisonByPeriod(String stationIdc, String period) {
        // 固定时段转时间范围
        LocalDateTime startTime = convertPeriodToTimeRange(period);
        return queryStationComparisonByTimeRange(stationIdc, startTime, LocalDateTime.now());
    }

    /**
     * 按时间范围查询多站对比
     *
     * @param stationIdc 站点编号（多个,隔开）
     * @param starttime  开始时间
     * @param endtime    结束时间
     * @return DataCompareView
     */
    @Override
    public DataCompareView queryStationComparisonByTimeRange(String stationIdc, LocalDateTime starttime, LocalDateTime endtime) {

        DataCompareView dataCompareView = new DataCompareView();
        ArrayList<DataCompare> dataCompareList = new ArrayList<>();
        ArrayList<HydroStationData> hydroStationDataList = new ArrayList<>();
        // 1、将 站点编号字符串 转换为List
        List<String> stationIdcList = Arrays.stream(stationIdc.replaceAll("，", ",").split(","))
                .collect(toList());
        // 2、依次循环查询每个站点的数据
        for (String eachS : stationIdcList) {
            List<HydroStationData> hydroStationData = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                    .eq(HydroStationData::getStcd, eachS)
                    .between(HydroStationData::getDt, df.format(starttime), df.format(endtime)));
            List<HydroStationDataValue> dataList = new ArrayList<>();
            for (HydroStationData each : hydroStationData) {
                dataList.add(HydroStationDataValue.builder().dt(df.format(each.getDt())).z(each.getZ()).q(each.getQ()).build());
            }
            dataCompareList.add(DataCompare.builder()
                    .stcd(eachS)
                    .data(dataList).build());
            hydroStationDataList.addAll(hydroStationData);
        }
        // 5、将数据按照 站点编号 时间 正序排列
        dataCompareList.sort(Comparator.comparing(DataCompare::getStcd));
        hydroStationDataList.sort(Comparator.comparing(HydroStationData::getStcd).thenComparing(HydroStationData::getDt));
        return DataCompareView.builder().
                dataCompares(dataCompareList).
                dataCompareList(hydroStationDataList).build();
    }

    /**
     * 按年查询历史值对比
     *
     * @param stationidc     站点编号
     * @param currentyear    当前年
     * @param comparisonyear 对比年
     * @param querytype      查询类型
     * @return DataCompareView
     */
    @Override
    public DataHisValueCompareView queryHistoryComparisonByYear(String stationidc, Integer currentyear, Integer comparisonyear, String querytype) {
        // 当前年的 流量 和 水位 List
        List<HydroMonthCharacterFlowValue> currFlowValueList = hydroMonthCharacterFlowValueMapper.selectList(new LambdaQueryWrapper<HydroMonthCharacterFlowValue>()
                .eq(HydroMonthCharacterFlowValue::getStcd, stationidc)
                .eq(HydroMonthCharacterFlowValue::getYr, currentyear));
        List<HydroMonthCharacterLevelValue> currLevelValueList = hydroMonthCharacterLevelValueMapper.selectList(new LambdaQueryWrapper<HydroMonthCharacterLevelValue>()
                .eq(HydroMonthCharacterLevelValue::getStcd, stationidc)
                .eq(HydroMonthCharacterLevelValue::getYr, currentyear));
        // 对比年的 流量 和 水位 List
        List<HydroMonthCharacterFlowValue> compFlowValueList = hydroMonthCharacterFlowValueMapper.selectList(new LambdaQueryWrapper<HydroMonthCharacterFlowValue>()
                .eq(HydroMonthCharacterFlowValue::getStcd, stationidc)
                .eq(HydroMonthCharacterFlowValue::getYr, comparisonyear));
        List<HydroMonthCharacterLevelValue> compLevelValueList = hydroMonthCharacterLevelValueMapper.selectList(new LambdaQueryWrapper<HydroMonthCharacterLevelValue>()
                .eq(HydroMonthCharacterLevelValue::getStcd, stationidc)
                .eq(HydroMonthCharacterLevelValue::getYr, comparisonyear));
        // 将当前年的 流量 和 水位 组合成 List<HydroStationDataValue>
        Map<Integer, HydroMonthCharacterFlowValue> currFlowMap = currFlowValueList.stream()
                .collect(Collectors.toMap(HydroMonthCharacterFlowValue::getMnth, f -> f));
        List<HydroStationDataValue> currDataList = currLevelValueList.stream()
                .map(l -> toHydroStationDataValue(currFlowMap.get(l.getMnth()), l)).collect(toList());
        // 将对比年的 流量 和 水位 组合成 List<HydroStationDataValue>
        Map<Integer, HydroMonthCharacterFlowValue> compFlowMap = compFlowValueList.stream()
                .collect(Collectors.toMap(HydroMonthCharacterFlowValue::getMnth, f -> f));
        List<HydroStationDataValue> compDataList = compLevelValueList.stream()
                .map(l -> toHydroStationDataValue(compFlowMap.get(l.getMnth()), l)).collect(toList());
        // 计算”距平“
        if (Constant.DIFF_AVG.equals(querytype)) {
            currDataList = new ArrayList<>(calculateDataValueAvg(currDataList, stationidc));
            compDataList = new ArrayList<>(calculateDataValueAvg(compDataList, stationidc));
        }
        // 组合成返回数据类
        ArrayList<DataCompare> compareList = new ArrayList<>();
        compareList.add(DataCompare.builder().time(currentyear).stcd(stationidc).data(currDataList).build());
        compareList.add(DataCompare.builder().time(comparisonyear).stcd(stationidc).data(compDataList).build());
        List<HydroStationDataValue> dataCompareListFull = new ArrayList<>();
        dataCompareListFull.addAll(currDataList);
        dataCompareListFull.addAll(compDataList);
        return DataHisValueCompareView.builder().dataCompares(compareList).dataCompareList(dataCompareListFull).build();
    }

    /**
     * 按月查询历史值对比
     *
     * @param stationidc      站点编号
     * @param currentmonth    当前年月
     * @param comparisonmonth 对比年月
     * @param querytype       查询类型
     * @return DataCompareView
     */
    @Override
    public DataHisValueCompareView queryHistoryComparisonByMonth(String stationidc, String currentmonth, String comparisonmonth, String querytype) {
        LocalDate currentmonth1 = LocalDate.parse(currentmonth + "-01");
        LocalDate comparisonmonth1 = LocalDate.parse(comparisonmonth + "-01");
        // 当前月和对比月的 水文特征值数据
        List<HydroDayCharacteristicValue> currDayValueList = hydroDayCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroDayCharacteristicValue>()
                .eq(HydroDayCharacteristicValue::getStcd, stationidc)
                .eq(HydroDayCharacteristicValue::getYr, currentmonth1.getYear())
                .eq(HydroDayCharacteristicValue::getMnth, currentmonth1.getMonthValue()));
        List<HydroDayCharacteristicValue> compDayValueList = hydroDayCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroDayCharacteristicValue>()
                .eq(HydroDayCharacteristicValue::getStcd, stationidc)
                .eq(HydroDayCharacteristicValue::getYr, comparisonmonth1.getYear())
                .eq(HydroDayCharacteristicValue::getMnth, comparisonmonth1.getMonthValue()));
        ArrayList<HydroStationDataValue> currDataList = new ArrayList<>();
        ArrayList<HydroStationDataValue> compDataList = new ArrayList<>();
        for (HydroDayCharacteristicValue e : currDayValueList) {
            currDataList.add(HydroStationDataValue.builder().dt(e.getYr() + "-" + String.format("%02d", e.getMnth()) + "-" + String.format("%02d", e.getDay())).time(e.getDay())
                    .htz(e.getHtz()).mxq(e.getMxq()).avz(e.getAvz()).avq(e.getAvq()).ltz(e.getLtz()).mnq(e.getMnq()).build());
        }
        for (HydroDayCharacteristicValue e : compDayValueList) {
            compDataList.add(HydroStationDataValue.builder().dt(e.getYr() + "-" + String.format("%02d", e.getMnth()) + "-" + String.format("%02d", e.getDay())).time(e.getDay())
                    .htz(e.getHtz()).mxq(e.getMxq()).avz(e.getAvz()).avq(e.getAvq()).ltz(e.getLtz()).mnq(e.getMnq()).build());
        }
        // 计算”距平“
        if (Constant.DIFF_AVG.equals(querytype)) {
            currDataList = new ArrayList<>(calculateDataValueAvg(currDataList, stationidc));
            compDataList = new ArrayList<>(calculateDataValueAvg(compDataList, stationidc));
        }
        // 组合成返回数据类
        ArrayList<DataCompare> compareList = new ArrayList<>();
        compareList.add(DataCompare.builder().time(currentmonth1.getMonth().getValue()).stcd(stationidc).data(currDataList).build());
        compareList.add(DataCompare.builder().time(comparisonmonth1.getMonth().getValue()).stcd(stationidc).data(compDataList).build());
        List<HydroStationDataValue> dataCompareListFull = new ArrayList<>();
        dataCompareListFull.addAll(currDataList);
        dataCompareListFull.addAll(compDataList);
        return DataHisValueCompareView.builder().dataCompares(compareList).dataCompareList(dataCompareListFull).build();
    }

    /**
     * 按日查询历史值对比
     *
     * @param stationidc    站点编号
     * @param currentday    当前年月日
     * @param comparisonday 对比年月日
     * @param querytype     查询类型
     * @return DataCompareView
     */
    @Override
    public DataHisValueCompareView queryHistoryComparisonByDay(String stationidc, LocalDate currentday, LocalDate comparisonday, String querytype) {

        List<HydroStationData> currStationData = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                .eq(HydroStationData::getStcd, stationidc)
                .between(HydroStationData::getDt, df.format(currentday.atStartOfDay()),
                        df.format(LocalDateTimeUtil.offset(currentday.atStartOfDay(), 1, ChronoUnit.DAYS))));
        List<HydroStationData> compStationData = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                .eq(HydroStationData::getStcd, stationidc)
                .between(HydroStationData::getDt, df.format(comparisonday.atStartOfDay()),
                        df.format(LocalDateTimeUtil.offset(comparisonday.atStartOfDay(), 1, ChronoUnit.DAYS))));
        ArrayList<HydroStationDataValue> currDataList = new ArrayList<>();
        ArrayList<HydroStationDataValue> compDataList = new ArrayList<>();
        for (HydroStationData e : currStationData) {
            currDataList.add(HydroStationDataValue.builder().dt(df.format(e.getDt())).time(e.getDt().getHour())
                    .z(e.getZ()).q(e.getQ()).build());
        }
        for (HydroStationData e : compStationData) {
            compDataList.add(HydroStationDataValue.builder().dt(df.format(e.getDt())).time(e.getDt().getHour())
                    .z(e.getZ()).q(e.getQ()).build());
        }
        // 计算”距平“
        if (Constant.DIFF_AVG.equals(querytype)) {
            HydroStationHistory stationHistory = hydroStationHistoryMapper.selectOne(new LambdaQueryWrapper<HydroStationHistory>()
                    .eq(HydroStationHistory::getStcd, stationidc));
            Double historyFlowAvg = stationHistory.getHavq();
            Double historyLevelAvg = stationHistory.getHavz();
            currDataList.forEach(each -> {
                each.setZ(each.getZ() - historyLevelAvg);
                each.setQ(each.getQ() - historyFlowAvg);
            });
            compDataList.forEach(each -> {
                each.setZ(each.getZ() - historyLevelAvg);
                each.setQ(each.getQ() - historyFlowAvg);
            });
        }
        // 组合成返回数据类
        ArrayList<DataCompare> compareList = new ArrayList<>();
        compareList.add(DataCompare.builder()
                .time(comparisonday.getDayOfMonth()).stcd(stationidc).data(currDataList).build());
        compareList.add(DataCompare.builder()
                .time(comparisonday.getDayOfMonth()).stcd(stationidc).data(compDataList).build());
        List<HydroStationDataValue> dataCompareListFull = new ArrayList<>();
        dataCompareListFull.addAll(currDataList);
        dataCompareListFull.addAll(compDataList);
        return DataHisValueCompareView.builder().dataCompares(compareList).dataCompareList(dataCompareListFull).build();
    }

    /**
     * 查询相似年对比
     *
     * @param stationIdc 站点编号
     * @return DataCompareView
     */
    @Override
    public DataSimilarView queryHistorySimilarityComparison(String stationIdc) {
        List<DataCompare> dataCompareList = new ArrayList<>();
        // 查询与 当前年 最相似的3年（逆序排序）
        List<HydroHistorySimilarValue> similarValueList = hydroHistorySimilarValueMapper.selectList(new LambdaQueryWrapper<HydroHistorySimilarValue>()
                .orderByDesc(HydroHistorySimilarValue::getSv)
                .last("LIMIT 3"));
        // 查询 当前年 的数据值（日统计）
        List<HydroStationDataValue> currDataValueList = new ArrayList<>();
        List<HydroDayCharacteristicValue> currDayValueList = hydroDayCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroDayCharacteristicValue>()
                .eq(HydroDayCharacteristicValue::getStcd, stationIdc)
                .eq(HydroDayCharacteristicValue::getYr, LocalDate.now().getYear())
                .between(HydroDayCharacteristicValue::getMnth, 1, 12)
                .between(HydroDayCharacteristicValue::getDay, 1, 31));
        currDayValueList.forEach(e -> {
            currDataValueList.add(HydroStationDataValue.builder().dt(e.getYr() + "-" + String.format("%02d", e.getMnth()) + "-" + String.format("%02d", e.getDay()))
                    .time(e.getYr()).z(e.getAvz()).q(e.getAvq()).build());
        });
        dataCompareList.add(DataCompare.builder().stcd(stationIdc).data(currDataValueList).build());
        List<HydroDayCharacteristicValue> dayCharacteristicValueList = new ArrayList<>(currDayValueList);
        // 依次循环查询 相似年 的数据值（日统计）
        for (HydroHistorySimilarValue each : similarValueList) {
            // 查询每一年数据并转存
            List<HydroStationDataValue> dataValueList = new ArrayList<>();
            List<HydroDayCharacteristicValue> dayValueList = hydroDayCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroDayCharacteristicValue>()
                    .eq(HydroDayCharacteristicValue::getStcd, stationIdc)
                    .eq(HydroDayCharacteristicValue::getYr, each.getYr())
                    .between(HydroDayCharacteristicValue::getMnth, 1, 12)
                    .between(HydroDayCharacteristicValue::getDay, 1, 31));
            dayValueList.forEach(e -> {
                dataValueList.add(HydroStationDataValue.builder().dt(e.getYr() + "-" + String.format("%02d", e.getMnth()) + "-" + String.format("%02d", e.getDay()))
                        .time(e.getYr()).z(e.getAvz()).q(e.getAvq()).build());
            });
            dataCompareList.add(DataCompare.builder().time(each.getYr()).stcd(stationIdc).data(dataValueList).build());
            dayCharacteristicValueList.addAll(dayValueList);
        }
        return DataSimilarView.builder().dataCompares(dataCompareList).similarValueList(similarValueList).dataCompareList(dayCharacteristicValueList).build();
    }

    /**
     * 按条件查询报汛特征值
     *
     * @param stationIdc 站号
     * @param startTime  开始时间
     * @param endTime    结束时间
     * @param query      查询类
     * @return List<HydroMonthCharacterValueView>
     */
    @Override
    public HydroMonthCharacterValueView queryMonthCharacteristicValue(String stationIdc, String startTime, String endTime, BasePageQuery query) {
        // 从数据库查数据（分页）
        Page<HydroMonthCharacterLevelValue> pageLevel = new Page<>(query.getCurrent(), query.getSize());
        Page<HydroMonthCharacterFlowValue> pageFlow = new Page<>(query.getCurrent(), query.getSize());
        hydroMonthCharacterLevelValueMapper.selectPage(pageLevel, new LambdaQueryWrapper<HydroMonthCharacterLevelValue>()
                .eq(HydroMonthCharacterLevelValue::getStcd, stationIdc)
                .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH),'%Y-%m-%d') BETWEEN '" + startTime + "-00" + "' AND '" + endTime + "-00" + "'"));
        hydroMonthCharacterFlowValueMapper.selectPage(pageFlow, new LambdaQueryWrapper<HydroMonthCharacterFlowValue>()
                .eq(HydroMonthCharacterFlowValue::getStcd, stationIdc)
                .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH),'%Y-%m-%d') BETWEEN '" + startTime + "-00" + "' AND '" + endTime + "-00" + "'"));
        // 拼接成大的List，组合成返回数据类（Map的key为”Yr-Mnth“格式的字符串）
        Map<String, HydroMonthCharacterFlowValue> flowMap = pageFlow.getRecords().stream()
                .collect(Collectors.toMap(f -> f.getYr() + "-" + f.getMnth(), f -> f));
        List<HydroMonthCharacterValue> dataList = pageLevel.getRecords().stream()
                .map(l -> toHydroMonthCharacterValue(flowMap.get(l.getYr() + "-" + l.getMnth()), l)).collect(toList());
        Page<HydroMonthCharacterValue> page = new Page<>(pageLevel.getCurrent(), pageLevel.getSize(), pageLevel.getTotal());
        page.setRecords(dataList);
        return HydroMonthCharacterValueView.builder().characterValue(new PageVo<>(page)).build();
    }

    /**
     * 按条件查询报汛特征值的历史对比
     *
     * @param stationIdc 站号
     * @param startTime  开始时间
     * @param endTime    结束时间
     * @param dataType   数据类型
     * @param query      查询类
     * @return HydroMonthCharacterValueView
     */
    @Override
    public HydroMonthCharacterValueCompareView queryMonthCharacteristicValueCompare(String stationIdc, String startTime, String endTime, String dataType, BasePageQuery query) {
        // 从数据库查实况数据（分页）
        Page<HydroMonthCharacterLevelValue> pageLevel = new Page<>(query.getCurrent(), query.getSize());
        Page<HydroMonthCharacterFlowValue> pageFlow = new Page<>(query.getCurrent(), query.getSize());
        hydroMonthCharacterLevelValueMapper.selectPage(pageLevel, new LambdaQueryWrapper<HydroMonthCharacterLevelValue>()
                .eq(HydroMonthCharacterLevelValue::getStcd, stationIdc)
                .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH),'%Y-%m-%d') BETWEEN '" +
                        startTime + "-00" + "' AND '" + endTime + "-00" + "'"));
        hydroMonthCharacterFlowValueMapper.selectPage(pageFlow, new LambdaQueryWrapper<HydroMonthCharacterFlowValue>()
                .eq(HydroMonthCharacterFlowValue::getStcd, stationIdc)
                .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH),'%Y-%m-%d') BETWEEN '" +
                        startTime + "-00" + "' AND '" + endTime + "-00" + "'"));
        // 从数据库查历史数据（分页）
        Page<HydroMonthCharacterLevelHistoryValue> pageLevelH = new Page<>(query.getCurrent(), query.getSize());
        Page<HydroMonthCharacterFlowHistoryValue> pageFlowH = new Page<>(query.getCurrent(), query.getSize());
        // 截取月份字符串 第5位到末尾（例如：2022-05  ->  5）
        Integer startMonth = Integer.valueOf(startTime.substring(5));
        Integer endMonth = Integer.valueOf(endTime.substring(5));
        hydroMonthCharacterLevelHistoryValueMapper.selectPage(pageLevelH, new LambdaQueryWrapper<HydroMonthCharacterLevelHistoryValue>()
                .eq(HydroMonthCharacterLevelHistoryValue::getStcd, stationIdc)
                .between(HydroMonthCharacterLevelHistoryValue::getMnth, startMonth, endMonth));
        hydroMonthCharacterFlowHistoryValueMapper.selectPage(pageFlowH, new LambdaQueryWrapper<HydroMonthCharacterFlowHistoryValue>()
                .eq(HydroMonthCharacterFlowHistoryValue::getStcd, stationIdc)
                .between(HydroMonthCharacterFlowHistoryValue::getMnth, startMonth, endMonth));
        // 拼接成大的List，组合成返回数据类（Map的key为”Yr-Mnth“格式的字符串）
        Map<String, HydroMonthCharacterFlowValue> flowMap = pageFlow.getRecords().stream()
                .collect(Collectors.toMap(f -> f.getYr() + "-" + f.getMnth(), f -> f));
        Map<Integer, HydroMonthCharacterLevelHistoryValue> levelHistoryMap = pageLevelH.getRecords().stream()
                .collect(Collectors.toMap(HydroMonthCharacterLevelHistoryValue::getMnth, lh -> lh));
        Map<Integer, HydroMonthCharacterFlowHistoryValue> flowHistoryMap = pageFlowH.getRecords().stream()
                .collect(Collectors.toMap(HydroMonthCharacterFlowHistoryValue::getMnth, fh -> fh));
        // 按条件组合返回类，并计算距平
        List<HydroMonthCharacterValueCompare> dataList = pageLevel.getRecords().stream().map(l -> toHydroMonthCharacterValueCompare(
                flowMap.get(l.getYr() + "-" + l.getMnth()), l, flowHistoryMap.get(l.getMnth()), levelHistoryMap.get(l.getMnth()), dataType, stationIdc
        )).collect(toList());
        Page<HydroMonthCharacterValueCompare> page = new Page<>(pageLevel.getCurrent(), pageLevel.getSize(), pageLevel.getTotal());
        page.setRecords(dataList);
        return HydroMonthCharacterValueCompareView.builder().compareData(new PageVo<>(page)).build();
    }

    /**
     * 多条件查询特征值排序信息
     *
     * @param stationIdc 站号
     * @param datatype   数据类型
     * @param period     对比周期
     * @param startTime  开始周期
     * @param endTime    结束周期
     * @param orderType  排序方式
     * @param querytype  查询类型（确认/距平）
     * @return CharacteristicValueOrderView
     */
    @Override
    public CharacteristicValueOrderView queryCharacteristicValueOrder(String stationIdc, String datatype, String period, String startTime, String endTime, String orderType, String querytype) {
        List<CharacteristicValueOrder> characteristicValueOrderList = new ArrayList<>();
        CharacteristicValueOrderView characteristicValueOrderView = new CharacteristicValueOrderView();
        HydroStationHistory hydroStationHistories = new HydroStationHistory();
        try {
            //根据站号获取站历史特征值列表
            QueryWrapper<HydroStationHistory> queryWrapper = new QueryWrapper<>();
            hydroStationHistories = hydroStationHistoryMapper.selectOne(queryWrapper.eq("STCD", stationIdc));

            switch (period) {
                case "年":
                    //根据站号数据类型和排序方式获取年特征值列表
                    characteristicValueOrderList = queryYearCharacteristicValueByParams(stationIdc, datatype, startTime, endTime, orderType, querytype, hydroStationHistories);
                    break;
                case "月":
                    //根据站号数据类型和排序方式获取月特征值列表
                    characteristicValueOrderList = queryMonthCharacteristicValueByParams(stationIdc, datatype, startTime, endTime, orderType, querytype, hydroStationHistories);
                    break;
                case "日":
                    //根据站号数据类型和排序方式获取天特征值列表
                    characteristicValueOrderList = queryDayCharacteristicValueByParams(stationIdc, datatype, startTime, endTime, orderType, querytype, hydroStationHistories);
                    break;
                default:
                    throw new Exception("请传入正确的对比周期，包含[年][月][日]");
            }


        } catch (Exception e) {
            log.error("多条件查询特征值排序信息信息失败！");
            e.printStackTrace();
        }
        characteristicValueOrderView.setCharacteristicValueOrderList(characteristicValueOrderList);
        return characteristicValueOrderView;
    }

    /**
     * 根据站号数据类型和排序方式获取年特征值列表
     *
     * @param stationIdc            站号
     * @param datatype              数据类型
     * @param startTime             开始周期
     * @param endTime               结束周期
     * @param orderType             排序方式
     * @param querytype             查询类型（确认/距平）
     * @param hydroStationHistories 历史特征值
     * @return List<CharacteristicValueOrder>
     */
    private List<CharacteristicValueOrder> queryYearCharacteristicValueByParams(String stationIdc, String datatype, String startTime, String endTime, String orderType, String querytype, HydroStationHistory hydroStationHistories) throws Exception {
        List<CharacteristicValueOrder> characteristicValueOrderList = new ArrayList<>();
        List<HydroYearCharacteristicValue> hydroYearCharacteristicValues = hydroYearCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroYearCharacteristicValue>()
                .eq(HydroYearCharacteristicValue::getStcd, stationIdc)
                .between(HydroYearCharacteristicValue::getYr, startTime, endTime));
        for (HydroYearCharacteristicValue each : hydroYearCharacteristicValues) {
            CharacteristicValueOrder characteristicValueOrder;
            //拼出时间字符串放进返回类
            String date = each.getYr() + "-00-00";
            Double value;
            Double differValue = null;
            Double distanceFlat = null;
            switch (datatype) {
                case (Constant.MAX_Z):
                    if (!ObjectUtils.isEmpty(each.getHtz())) {
                        differValue = each.getHtz();
                        distanceFlat = each.getHtz() - hydroStationHistories.getHhtz();
                    }
                    break;
                case (Constant.MIN_Z):
                    if (!ObjectUtils.isEmpty(each.getLtz())) {
                        differValue = each.getLtz();
                        distanceFlat = each.getLtz() - hydroStationHistories.getHltz();
                    }
                    break;
                case (Constant.AVG_Z):
                    if (!ObjectUtils.isEmpty(each.getAvz())) {
                        differValue = each.getAvz();
                        distanceFlat = each.getAvz() - hydroStationHistories.getHavz();
                    }
                    break;
                case (Constant.MAX_Q):
                    if (!ObjectUtils.isEmpty(each.getMxq())) {
                        differValue = each.getMxq();
                        distanceFlat = each.getMxq() - hydroStationHistories.getHmxq();
                    }
                    break;
                case (Constant.MIN_Q):
                    if (!ObjectUtils.isEmpty(each.getMnq())) {
                        differValue = each.getMnq();
                        distanceFlat = each.getMnq() - hydroStationHistories.getHmnq();
                    }
                    break;
                case (Constant.AVG_Q):
                    if (!ObjectUtils.isEmpty(each.getAvq())) {
                        differValue = each.getAvq();
                        distanceFlat = each.getAvq() - hydroStationHistories.getHavq();
                    }
                    break;
                default:
                    throw new Exception("请传入正确的对比周期，包含[最高水位][最低水位][平均水位][最大流量][最小流量][平均流量]");
            }
            if (!Constant.DIFF_AVG.equals(querytype)) {
                value = differValue;
            } else {
                value = distanceFlat;
            }
            characteristicValueOrder = CharacteristicValueOrder.builder().dt(date).value(value).build();
            characteristicValueOrderList.add(characteristicValueOrder);
        }
        //对获取的List排序
        return orderCharacteristicValueOrderList(orderType, characteristicValueOrderList);
    }

    /**
     * //对获取的List排序
     *
     * @param orderType
     * @param characteristicValueOrderList
     * @return List<CharacteristicValueOrder>
     */
    private List<CharacteristicValueOrder> orderCharacteristicValueOrderList(String orderType, List<CharacteristicValueOrder> characteristicValueOrderList) throws Exception {
        List<CharacteristicValueOrder> list;
        switch (orderType) {
            case ("时间排序"):
                list = characteristicValueOrderList.stream().sorted(Comparator.comparing(CharacteristicValueOrder::getDt, Comparator.nullsLast(String::compareTo)).reversed()).collect(toList());
                break;
            case ("数据排序"):
                list = characteristicValueOrderList.stream().sorted(Comparator.comparing(CharacteristicValueOrder::getValue, Comparator.nullsLast(Double::compareTo)).reversed()).collect(toList());
                break;
            default:
                throw new Exception("请传入正确的对比周期，包含[时间排序][数据排序]");
        }
        return list;
    }


    /**
     * 根据站号数据类型和排序方式获取月特征值列表
     *
     * @param stationIdc            站号
     * @param datatype              数据类型
     * @param startTime             开始周期
     * @param endTime               结束周期
     * @param orderType             排序方式
     * @param querytype             查询类型（确认/距平）
     * @param hydroStationHistories 历史特征值
     * @return List<CharacteristicValueOrder>
     */
    private List<CharacteristicValueOrder> queryMonthCharacteristicValueByParams(String stationIdc, String datatype, String startTime, String endTime, String orderType, String querytype, HydroStationHistory hydroStationHistories) throws Exception {
        List<CharacteristicValueOrder> characteristicValueOrderList = new ArrayList<>();
        if (datatype.contains("流量")) {
            //获取水文月特征流量值
            List<HydroMonthCharacterFlowValue> hydroMonthCharacterFlowValues = hydroMonthCharacterFlowValueMapper.selectList(new LambdaQueryWrapper<HydroMonthCharacterFlowValue>()
                    .eq(HydroMonthCharacterFlowValue::getStcd, stationIdc)
                    .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH),'%Y-%m-%d') BETWEEN '" +
                            startTime + "-00" + "' AND '" + endTime + "-00" + "'"));
            for (HydroMonthCharacterFlowValue each : hydroMonthCharacterFlowValues) {
                CharacteristicValueOrder characteristicValueOrder;
                //拼出时间字符串放进返回类
                String date = each.getYr() + each.getMnth() + "-00";
                Double value;
                Double differValue = (double) 0;
                Double distanceFlat = (double) 0;
                switch (datatype) {
                    case (Constant.MAX_Q):
                        if (!ObjectUtils.isEmpty(each.getMxq())) {
                            differValue = each.getMxq();
                            distanceFlat = each.getMxq() - hydroStationHistories.getHmxq();
                        }
                        break;
                    case (Constant.MIN_Q):
                        if (!ObjectUtils.isEmpty(each.getMnq())) {
                            differValue = each.getMnq();
                            distanceFlat = each.getMnq() - hydroStationHistories.getHmnq();
                        }
                        break;
                    case (Constant.AVG_Q):
                        if (!ObjectUtils.isEmpty(each.getAvq())) {
                            differValue = each.getAvq();
                            distanceFlat = each.getAvq() - hydroStationHistories.getHavq();
                        }
                        break;
                    default:
                        throw new Exception("请传入正确的对比周期，包含[最高水位][最低水位][平均水位][最大流量][最小流量][平均流量]");
                }
                if (!Constant.DIFF_AVG.equals(querytype)) {
                    value = differValue;
                } else {
                    value = distanceFlat;
                }
                characteristicValueOrder = CharacteristicValueOrder.builder().dt(date).value(value).build();
                characteristicValueOrderList.add(characteristicValueOrder);
            }
        } else {
            //获取水文月特征水位值
            List<HydroMonthCharacterLevelValue> hydroMonthCharacterLevelValues = hydroMonthCharacterLevelValueMapper.selectList(new LambdaQueryWrapper<HydroMonthCharacterLevelValue>()
                    .eq(HydroMonthCharacterLevelValue::getStcd, stationIdc)
                    .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH),'%Y-%m-%d') BETWEEN '" +
                            startTime + "-00" + "' AND '" + endTime + "-00" + "'"));
            for (HydroMonthCharacterLevelValue each : hydroMonthCharacterLevelValues) {
                CharacteristicValueOrder characteristicValueOrder;
                //拼出时间字符串放进返回类
                String month;
                if (each.getMnth() < 10) {
                    month = "0" + each.getMnth();
                } else {
                    month = String.valueOf(each.getMnth());
                }
                String date = each.getYr() + "-" + month + "-00";
                Double value;
                Double differValue = (double) 0;
                Double distanceFlat = (double) 0;
                switch (datatype) {
                    case (Constant.MAX_Z):
                        if (!ObjectUtils.isEmpty(each.getHtz())) {
                            differValue = each.getHtz();
                            distanceFlat = each.getHtz() - hydroStationHistories.getHhtz();
                        }
                        break;
                    case (Constant.MIN_Z):
                        if (!ObjectUtils.isEmpty(each.getLtz())) {
                            differValue = each.getLtz();
                            distanceFlat = each.getLtz() - hydroStationHistories.getHltz();
                        }
                        break;
                    case (Constant.AVG_Z):
                        if (!ObjectUtils.isEmpty(each.getAvz())) {
                            differValue = each.getAvz();
                            distanceFlat = each.getAvz() - hydroStationHistories.getHavz();
                        }
                        break;
                    default:
                        throw new Exception("请传入正确的对比周期，包含[最高水位][最低水位][平均水位][最大流量][最小流量][平均流量]");
                }
                if (!Constant.DIFF_AVG.equals(querytype)) {
                    value = differValue;
                } else {
                    value = distanceFlat;
                }
                characteristicValueOrder = CharacteristicValueOrder.builder().dt(date).value(value).build();
                characteristicValueOrderList.add(characteristicValueOrder);
            }
        }
        //对获取的List排序
        return orderCharacteristicValueOrderList(orderType, characteristicValueOrderList);
    }


    /**
     * 根据站号数据类型和排序方式获取日特征值列表
     *
     * @param stationIdc            站号
     * @param datatype              数据类型
     * @param startTime             开始周期
     * @param endTime               结束周期
     * @param orderType             排序方式
     * @param querytype             查询类型（确认/距平）
     * @param hydroStationHistories 历史特征值
     * @return List<CharacteristicValueOrder>
     */
    private List<CharacteristicValueOrder> queryDayCharacteristicValueByParams(String stationIdc, String datatype, String startTime, String endTime, String orderType, String querytype, HydroStationHistory hydroStationHistories) throws Exception {
        List<CharacteristicValueOrder> characteristicValueOrderList = new ArrayList<>();
        //获取
        List<HydroDayCharacteristicValue> hydroDayCharacteristicValues = hydroDayCharacteristicValueMapper.selectList(new LambdaQueryWrapper<HydroDayCharacteristicValue>()
                .eq(HydroDayCharacteristicValue::getStcd, stationIdc)
                .apply("STR_TO_DATE(CONCAT(YR, '-', MNTH , '-' , DAY),'%Y-%m-%d') BETWEEN '" +
                        startTime + "' AND '" + endTime + "'"));
        for (HydroDayCharacteristicValue each : hydroDayCharacteristicValues) {
            CharacteristicValueOrder characteristicValueOrder;
            //拼出时间字符串放进返回类
            String month;
            String day;
            if (each.getMnth() < 10) {
                month = "0" + each.getMnth();
            } else {
                month = String.valueOf(each.getMnth());
            }
            if (each.getDay() < 10) {
                day = "0" + each.getDay();
            } else {
                day = String.valueOf(each.getDay());
            }
            String date = each.getYr() + "-" + month + "-" + day;
            Double value;
            Double differValue = (double) 0;
            Double distanceFlat = (double) 0;
            switch (datatype) {
                case (Constant.MAX_Z):
                    if (!ObjectUtils.isEmpty(each.getHtz())) {
                        differValue = each.getHtz();
                        distanceFlat = each.getHtz() - hydroStationHistories.getHhtz();
                    }
                    break;
                case (Constant.MIN_Z):
                    if (!ObjectUtils.isEmpty(each.getLtz())) {
                        differValue = each.getLtz();
                        distanceFlat = each.getLtz() - hydroStationHistories.getHltz();
                    }
                    break;
                case (Constant.AVG_Z):
                    if (!ObjectUtils.isEmpty(each.getAvz())) {
                        differValue = each.getAvz();
                        distanceFlat = each.getAvz() - hydroStationHistories.getHavz();
                    }
                    break;
                case (Constant.MAX_Q):
                    if (!ObjectUtils.isEmpty(each.getMxq())) {
                        differValue = each.getMxq();
                        distanceFlat = each.getMxq() - hydroStationHistories.getHmxq();
                    }
                    break;
                case (Constant.MIN_Q):
                    if (!ObjectUtils.isEmpty(each.getMnq())) {
                        differValue = each.getMnq();
                        distanceFlat = each.getMnq() - hydroStationHistories.getHmnq();
                    }
                    break;
                case (Constant.AVG_Q):
                    if (!ObjectUtils.isEmpty(each.getAvq())) {
                        differValue = each.getAvq();
                        distanceFlat = each.getAvq() - hydroStationHistories.getHavq();
                    }
                    break;
                default:
                    throw new Exception("请传入正确的对比周期，包含[最高水位][最低水位][平均水位][最大流量][最小流量][平均流量]");
            }
            if (!Constant.DIFF_AVG.equals(querytype)) {
                value = differValue;
            } else {
                value = distanceFlat;
            }
            characteristicValueOrder = CharacteristicValueOrder.builder().dt(date).value(value).build();
            characteristicValueOrderList.add(characteristicValueOrder);
        }
        //对获取的List排序
        return orderCharacteristicValueOrderList(orderType, characteristicValueOrderList);
    }


    /**
     * 按照时间查询水位流量关系信息
     *
     * @param stationIdc 站号
     * @param year       统计年
     * @param month      统计月
     * @return HydroStationDataView
     */
    @Override
    public HydroStationDataView queryHydroStationDataList(String stationIdc, Integer year, Integer month) {
        HydroStationDataView hydroCompositionView = new HydroStationDataView();
        List<HydroStationDataInfo> hydroStationDataInfos = new ArrayList<>();
        List<HydroStationData> hydroStationDataList;
        HydroStationInfo hydroStationInfo;
        try {
            //按stationIdc查出水文站信息
            hydroStationInfo = hydroStationInfoMapper.selectOne(Wrappers.lambdaQuery(new HydroStationInfo())
                    .eq(HydroStationInfo::getStcd, stationIdc));
            //按时间和站号查出水文站数据
            hydroStationDataList = hydroStationDataMapper.selectList((new LambdaQueryWrapper<HydroStationData>()
                    .eq(HydroStationData::getStcd, stationIdc)
                    .apply("YEAR(DT) =" + year + " AND MONTH(DT) =" + month)));
            //讲查出的数据拼起来
            for (HydroStationData each : hydroStationDataList) {
                if (each.getStcd().equals(hydroStationInfo.getStcd())) {
                    //拼成HydroStationDataInfo
                    HydroStationDataInfo hydroStationDataInfo = HydroStationDataInfo.builder().dt(each.getDt()).stcd(stationIdc)
                            .stnm(hydroStationInfo.getStnm()).lgtd(hydroStationInfo.getLgtd()).lttd(hydroStationInfo.getLttd())
                            .z(each.getZ()).q(each.getQ()).zc(each.getZc()).qc(each.getQc()).zs(each.getZs()).qs(each.getQs())
                            .zcs(each.getZcs()).qcs(each.getQcs()).bywrz(each.getBywrz()).bygrz(each.getBygrz()).bywrq(each.getBywrq())
                            .bygrq(each.getBygrq()).build();
                    hydroStationDataInfos.add(hydroStationDataInfo);
                }
            }
            hydroCompositionView.setHydroStationDataList(hydroStationDataInfos);
        } catch (Exception e) {
            log.error("按照时间查询水位流量关系信息失败！");
            e.printStackTrace();
        }
        return hydroCompositionView;
    }

    /**
     * 按照时间范围查询雨量过程信息
     *
     * @param stationIdc 站号
     * @param stTime     开始时间
     * @param endTime    结束时间
     * @return RainfallProcessView
     */
    @Override
    public RainfallProcessView queryRainfallProcessListByTimeRange(String stationIdc, LocalDateTime stTime, LocalDateTime endTime) {
        List<MeteRainfallProcess> meteRainfallProcesses = new LinkedList<>();
        try {
            List<MeteRainfall> rainfallProcessList = meteRainfallProcessMapper.selectList(new LambdaQueryWrapper<MeteRainfall>()
                    .eq(MeteRainfall::getStcd, stationIdc)
                    .between(MeteRainfall::getDt, df.format(stTime), df.format(LocalDateTime.now())));
            Double accp = 0.0;
            for (MeteRainfall meteRainFall : rainfallProcessList
            ) {
                accp = accp + meteRainFall.getP();
                meteRainfallProcesses.add(MeteRainfallProcess.builder().stcd(meteRainFall.getStcd()).dt(meteRainFall.getDt()).p(meteRainFall.getP()).accup(accp).build());
            }
        } catch (Exception e) {
            log.error("按条件查询雨量过程信息失败！");
            e.printStackTrace();
        }
        return RainfallProcessView.builder().rainfallProcessList(meteRainfallProcesses).build();
    }

    /**
     * 按照时间范围和流域查询流域站点面雨量信息
     *
     * @param stTime   开始时间
     * @param endTime  结束时间
     * @param bsidList 流域id字符串列表
     * @return WatershedStationArealRainfallView
     */
    @Override
    public WatershedStationArealRainfallView queryWatershedStationArealRainfallList(LocalDateTime stTime, LocalDateTime endTime, String bsidList) {
        List<WatershedStationArealRainfall> watershedStationArealRainfallList;
        WatershedStationArealRainfallView watershedStationArealRainfallView = new WatershedStationArealRainfallView();
        try {
            String bsid = bsidList.replaceAll("，", ",");
            watershedStationArealRainfallList = watershedStationArealRainfallMapper.selectList(new LambdaQueryWrapper<WatershedStationArealRainfall>()
                    .between(WatershedStationArealRainfall::getDt, df.format(stTime), df.format(LocalDateTime.now()))
                    .apply("FIND_IN_SET(BSID," + "'" + bsid + "')"));
            watershedStationArealRainfallView.setWatershedStationArealRainfallList(watershedStationArealRainfallList);
        } catch (Exception e) {
            log.error("按条件查询流域站点面雨量信息失败！");
            e.printStackTrace();
        }
        return watershedStationArealRainfallView;
    }

    @Override
    public PageVo<MeteStationData> queryMeteStationDataByTimeAndBsid(LocalDateTime time, String bsid) {
        return null;
    }

    @Override
    public PageVo<MeteStationForcastingData> queryMeteStationForecastDataByTimeAndBsid(LocalDateTime dataTime, Integer validTime, String bsid, String type) {
        return null;
    }


    /**
     * 查询最新流域气象预报降水信息
     *
     * @return WatershedMeteForecastView
     */
    @Override
    public WatershedMeteForecastView queryWatershedMeteForecastList(LocalDateTime dataTime, Integer validTime) {
        try {
            LambdaQueryWrapper<WatershedMeteForecast> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(WatershedMeteForecast::getDt, dataTime).eq(WatershedMeteForecast::getValidtm, validTime);
            List<WatershedMeteForecast> watershedMeteForecasts = watershedMeteForecastMapper.selectList(lambdaQueryWrapper);
            return WatershedMeteForecastView.builder().watershedMeteForecastList(watershedMeteForecasts).build();
        } catch (Exception e) {
            log.error("查询最新流域气象预报信息失败！");
            return null;
        }
    }

    /**
     * 查询测站信息
     *
     * @param stationIdc 站号
     * @return HydroStationInfo
     */
    @Override
    public HydroStationInfo queryHydroStationInfo(String stationIdc) {
        HydroStationInfo hydroStationInfo = new HydroStationInfo();
        try {
            hydroStationInfo = hydroStationInfoMapper.selectOne(new LambdaQueryWrapper<HydroStationInfo>()
                    .eq(HydroStationInfo::getStcd, stationIdc));
        } catch (Exception e) {
            log.error("查询测站信息失败！");
            e.printStackTrace();
        }
        return hydroStationInfo;
    }


    /**
     * 按条件查询径流组成
     *
     * @param stationIdc 站点编号
     * @param mid
     * @param dateTime
     * @param validTime
     * @return
     */
    @Override
    public HydroCompositionView queryHydroCompositionByPeriod(String stationIdc, Integer mid, LocalDateTime dateTime, Integer validTime) {
        LambdaQueryWrapper<HydroComposition> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(HydroComposition::getDt, dateTime).eq(HydroComposition::getMid, mid).eq(HydroComposition::getValidtm, validTime).eq(HydroComposition::getStcd, stationIdc);
        HydroComposition hydroComposition = hydroCompositionMapper.selectOne(lambdaQueryWrapper);
        return HydroCompositionView.builder().
                gi(hydroComposition.getGi()).
                stcd(hydroComposition.getStcd()).
                gr(hydroComposition.getGr()).
                ri(hydroComposition.getRi()).
                rr(hydroComposition.getRr()).
                si(hydroComposition.getSi()).
                sr(hydroComposition.getSr()).
                build();
    }

//    /**
//     * 按时间范围查询径流组成信息
//     *
//     * @param stationIdc 站号
//     * @param stTime     开始时间
//     * @param endTime    结束时间
//     * @return HydroComposition
//     */
//    @Override
//    public HydroCompositionView queryHydroCompositionByTimeRange(String stationIdc, LocalDateTime stTime, LocalDateTime endTime) {
//        List<HydroComposition> hydroCompositions = new ArrayList<>();
//        HydroCompositionView hydroCompositionView = new HydroCompositionView();
//        try {
//            //查出时间段的list数据
//            hydroCompositions = hydroCompositionMapper.selectList(new LambdaQueryWrapper<HydroComposition>()
//                    .eq(HydroComposition::getStcd, stationIdc)
//                    .between(HydroComposition::getDt, df.format(stTime), df.format(LocalDateTime.now())));
//            //求得平均值
//            if (ObjectUtils.isEmpty(hydroCompositions)) {
//                Double giAvg = hydroCompositions.stream().mapToDouble(HydroComposition::getGi).average().getAsDouble();
//                Double grAvg = hydroCompositions.stream().mapToDouble(HydroComposition::getRi).average().getAsDouble();
//                Double siAvg = hydroCompositions.stream().mapToDouble(HydroComposition::getSi).average().getAsDouble();
//                Double srAvg = hydroCompositions.stream().mapToDouble(HydroComposition::getSr).average().getAsDouble();
//                Double riAvg = hydroCompositions.stream().mapToDouble(HydroComposition::getRi).average().getAsDouble();
//                Double rrAvg = hydroCompositions.stream().mapToDouble(HydroComposition::getRr).average().getAsDouble();
//                //拼成径流组成信息
//                hydroCompositionView = HydroCompositionView.builder().stcd(stationIdc).gi(giAvg).gr(grAvg).si(siAvg).sr(srAvg).ri(riAvg).rr(rrAvg).build();
//            }
//        } catch (Exception e) {
//            log.error("按条件查询径流组成信息失败！");
//            e.printStackTrace();
//        }
//        return hydroCompositionView;
//    }

    /**
     * 按照固定时段查询雨量过程信息
     *
     * @param stationIdc 站点编号
     * @param period     时段
     * @return RainfallProcessView
     */
    @Override
    public RainfallProcessView queryRainfallProcessListByPeriod(String stationIdc, String period) {
        LocalDateTime startTime = convertPeriodToTimeRange(period);
        return queryRainfallProcessListByTimeRange(stationIdc, startTime, LocalDateTime.now());
    }

    /**
     * 按照固定时段和流域查询流域站点面雨量信息
     *
     * @param period   时段
     * @param bsidList 流域id字符串列表
     * @return WatershedStationArealRainfallView
     */
    @Override
    public WatershedStationArealRainfallView queryWatershedStationArealRainfallListByPeriod(String period, String bsidList) {
        LocalDateTime sarttime = convertPeriodToTimeRange(period);
        String bsid = bsidList.replaceAll("，", ",");
        return queryWatershedStationArealRainfallList(sarttime, LocalDateTime.now(), bsid);
    }

    /**
     * 固定时段转时间范围
     *
     * @param period 固定时段
     * @return startTime 时间范围的开始时间
     */
    private LocalDateTime convertPeriodToTimeRange(String period) {
        // 按照条件推算 开始时间
        if (period.equals(TimeLast.LAST_12_HOUR.getName())) {
            return DateUtil.offsetHour(new Date(), -12).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_24_HOUR.getName())) {
            return DateUtil.offsetHour(new Date(), -24).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_3_DAY.getName())) {
            return DateUtil.offsetDay(new Date(), -3).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_1_WEEK.getName())) {
            return DateUtil.offsetWeek(new Date(), -1).toLocalDateTime();
        } else if (period.equals(TimeLast.LAST_1_MONTH.getName())) {
            return DateUtil.offsetMonth(new Date(), -1).toLocalDateTime();
        } else {
            throw new ParamsException("指定固定时段错误！可用固定时段：" + Arrays.stream(TimeLast.values())
                    .map(TimeLast::getName).collect(Collectors.joining("，")));
        }
    }

    /**
     * 判断并处理开始时间跨年问题
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return LocalDateTime 处理后的开始时间
     */
    private LocalDateTime handleOverYear(LocalDateTime startTime, LocalDateTime endTime) {
        // 确保开始时间不跨年
        LocalDateTime dataTimeOfTheYear = LocalDateTime.of(endTime.getYear(), 1, 1, 0, 0, 0);
        if (startTime.isBefore(dataTimeOfTheYear)) {
            return dataTimeOfTheYear;
        } else {
            return startTime;
        }
    }


    /**
     * 对数据进行距平
     *
     * @param dataValueList 水文站点数据
     * @param stationidc    站号
     * @return List<HydroStationDataValue> 距平后的水文站点数据
     */
    private List<HydroStationDataValue> calculateDataValueAvg(List<HydroStationDataValue> dataValueList, String stationidc) {
        HydroStationHistory stationHistory = hydroStationHistoryMapper.selectOne(new LambdaQueryWrapper<HydroStationHistory>()
                .eq(HydroStationHistory::getStcd, stationidc));
        Double historyFlowAvg = stationHistory.getHavq();
        Double historyLevelAvg = stationHistory.getHavz();
        dataValueList.forEach(each -> {
            each.setHtz(each.getHtz() - historyLevelAvg);
            each.setMxq(each.getMxq() - historyFlowAvg);
            each.setAvz(each.getAvz() - historyLevelAvg);
            each.setAvq(each.getAvq() - historyFlowAvg);
            each.setLtz(each.getLtz() - historyLevelAvg);
            each.setMnq(each.getMnq() - historyFlowAvg);
        });
        return dataValueList;
    }

    /**
     * 将水文 月份 【流量特征值】 和 【水位特征值】 进行组合
     *
     * @param flow  流量值
     * @param level 水位值
     * @return HydroStationDataValue
     */
    private HydroStationDataValue toHydroStationDataValue(HydroMonthCharacterFlowValue flow, HydroMonthCharacterLevelValue level) {
        return HydroStationDataValue.builder().dt(flow.getYr() + "-" + String.format("%02d", flow.getMnth())).time(flow.getMnth())
                .htz(level.getHtz()).mxq(flow.getMxq())
                .avz(level.getAvz()).avq(flow.getAvq())
                .ltz(level.getLtz()).mnq(flow.getMnq())
                .build();
    }

    /**
     * 将水文 月份 【流量特征值】 和 【水位特征值】组合成 报汛特征值-月特征值返回类
     *
     * @param flow  流量值
     * @param level 水位值
     * @return HydroMonthCharacterValue
     */
    private HydroMonthCharacterValue toHydroMonthCharacterValue(HydroMonthCharacterFlowValue flow, HydroMonthCharacterLevelValue level) {
        // 分别组合 上旬 中旬 下旬 当月 的特征值
        CharacterValue earlyVal = CharacterValue.builder()
                .htz(level.getHtze()).ltz(level.getLtze()).avz(level.getAvze())
                .mxq(flow.getMxqe()).mnq(flow.getMnqe()).avq(flow.getAvqe())
                .build();
        CharacterValue middleVal = CharacterValue.builder()
                .htz(level.getHtzm()).ltz(level.getLtzm()).avz(level.getAvzm())
                .mxq(flow.getMxqm()).mnq(flow.getMnqm()).avq(flow.getAvqm())
                .build();
        CharacterValue lateVal = CharacterValue.builder()
                .htz(level.getHtzl()).ltz(level.getLtzl()).avz(level.getAvzl())
                .mxq(flow.getMxql()).mnq(flow.getMnql()).avq(flow.getAvql())
                .build();
        CharacterValue currentVal = CharacterValue.builder()
                .htz(level.getHtz()).ltz(level.getLtz()).avz(level.getAvz())
                .mxq(flow.getMxq()).mnq(flow.getMnq()).avq(flow.getAvq())
                .build();
        return HydroMonthCharacterValue.builder()
                .stcd(flow.getStcd()).yr(flow.getYr()).mnth(flow.getMnth())
                .early(earlyVal).middle(middleVal).late(lateVal).current(currentVal)
                .build();
    }

    /**
     * 将水文 月份 【流量值】【水位值】【流量历史值】【水位历史值】组合成 报汛特征值-月历史对比返回类
     *
     * @param flow         流量值
     * @param level        水位值
     * @param flowHistory  流量历史值
     * @param levelHistory 水位历史值
     * @param dataType     数据类型
     * @param stationIdc   站点编号
     * @return HydroMonthCharacterValueCompare
     */
    private HydroMonthCharacterValueCompare toHydroMonthCharacterValueCompare(HydroMonthCharacterFlowValue flow, HydroMonthCharacterLevelValue level,
                                                                              HydroMonthCharacterFlowHistoryValue flowHistory, HydroMonthCharacterLevelHistoryValue levelHistory,
                                                                              String dataType, String stationIdc) {
        CharacterCompareValue earlyVal = null, middleVal = null, lateVal = null, currentVal = null;
        // 获取 距平 计算数据（站点 有史以来 平均流量和水位）
        HydroStationHistory stationHistory = hydroStationHistoryMapper.selectOne(new LambdaQueryWrapper<HydroStationHistory>()
                .eq(HydroStationHistory::getStcd, stationIdc));
        Double historyFlowAvg = stationHistory.getHavq();
        Double historyLevelAvg = stationHistory.getHavz();
        // 计算 距平，并按照 数据类型【最大 最小 平均】赋值
        if (Constant.MAX.equals(dataType)) {
            // 【上旬、中旬、下旬和当月】的<最大> 水位 和 流量
            earlyVal = CharacterCompareValue.builder()
                    .cz(level.getHtze()).hz(levelHistory.getHhtze()).az(level.getHtze() - historyLevelAvg)
                    .cq(flow.getMxqe()).hq(flowHistory.getHmxqe()).aq(flow.getMxqe() - historyFlowAvg).build();
            middleVal = CharacterCompareValue.builder()
                    .cz(level.getHtzm()).hz(levelHistory.getHhtzm()).az(level.getHtzm() - historyLevelAvg)
                    .cq(flow.getMxqm()).hq(flowHistory.getHmxqm()).aq(flow.getMxqm() - historyFlowAvg).build();
            lateVal = CharacterCompareValue.builder()
                    .cz(level.getHtzl()).hz(levelHistory.getHhtzl()).az(level.getHtzl() - historyLevelAvg)
                    .cq(flow.getMxql()).hq(flowHistory.getHmxql()).aq(flow.getMxql() - historyFlowAvg).build();
            currentVal = CharacterCompareValue.builder()
                    .cz(level.getHtz()).hz(levelHistory.getHhtz()).az(level.getHtz() - historyLevelAvg)
                    .cq(flow.getMxq()).hq(flowHistory.getHmxq()).aq(flow.getMxq() - historyFlowAvg).build();
        } else if (Constant.MIN.equals(dataType)) {
            // 【上旬、中旬、下旬和当月】的<最小> 水位 和 流量
            earlyVal = CharacterCompareValue.builder()
                    .cz(level.getLtze()).hz(levelHistory.getHltze()).az(level.getLtze() - historyLevelAvg)
                    .cq(flow.getMnqe()).hq(flowHistory.getHmnqe()).aq(flow.getMnqe() - historyFlowAvg).build();
            middleVal = CharacterCompareValue.builder()
                    .cz(level.getLtzm()).hz(levelHistory.getHltzm()).az(level.getLtzm() - historyLevelAvg)
                    .cq(flow.getMnqm()).hq(flowHistory.getHmnqm()).aq(flow.getMnqm() - historyFlowAvg).build();
            lateVal = CharacterCompareValue.builder()
                    .cz(level.getLtzl()).hz(levelHistory.getHltzl()).az(level.getLtzl() - historyLevelAvg)
                    .cq(flow.getMnql()).hq(flowHistory.getHmnql()).aq(flow.getMnql() - historyFlowAvg).build();
            currentVal = CharacterCompareValue.builder()
                    .cz(level.getLtz()).hz(levelHistory.getHltz()).az(level.getLtz() - historyLevelAvg)
                    .cq(flow.getMnq()).hq(flowHistory.getHmnq()).aq(flow.getMnq() - historyFlowAvg).build();
        } else if (Constant.AVG.equals(dataType)) {
            // 【上旬、中旬、下旬和当月】的<平均> 水位 和 流量
            earlyVal = CharacterCompareValue.builder()
                    .cz(level.getAvze()).hz(levelHistory.getHavze()).az(level.getAvze() - historyLevelAvg)
                    .cq(flow.getAvqe()).hq(flowHistory.getHavqe()).aq(flow.getAvqe() - historyFlowAvg).build();
            middleVal = CharacterCompareValue.builder()
                    .cz(level.getAvzm()).hz(levelHistory.getHavzm()).az(level.getAvzm() - historyLevelAvg)
                    .cq(flow.getMxqm()).hq(flowHistory.getHmxqm()).aq(flow.getMxqm() - historyFlowAvg).build();
            lateVal = CharacterCompareValue.builder()
                    .cz(level.getAvzl()).hz(levelHistory.getHavzl()).az(level.getAvzl() - historyLevelAvg)
                    .cq(flow.getMxql()).hq(flowHistory.getHmxql()).aq(flow.getMxql() - historyFlowAvg).build();
            currentVal = CharacterCompareValue.builder()
                    .cz(level.getAvz()).hz(levelHistory.getHavz()).az(level.getAvz() - historyLevelAvg)
                    .cq(flow.getMxq()).hq(flowHistory.getHmxq()).aq(flow.getMxq() - historyFlowAvg).build();
        } else {
            throw new ErrorMessage("请传入正确的数据类型，包含[最大][最小][平均]");
        }
        // 组合成一个类返回
        return HydroMonthCharacterValueCompare.builder()
                .stcd(flow.getStcd()).yr(flow.getYr()).mnth(flow.getMnth())
                .early(earlyVal).middle(middleVal).late(lateVal).current(currentVal)
                .build();
    }
}
