package com.cuit.basin.base.biz.service.model;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.cuit.basin.base.biz.controller.model.vo.InitTaskParams;
import com.cuit.basin.base.biz.controller.model.vo.ModelTaskInfo;
import com.cuit.basin.base.biz.controller.model.vo.SuccessTaskParams;
import com.cuit.basin.base.biz.convert.ModelLogConvert;
import com.cuit.basin.base.biz.dao.ModelLogMapper;
import com.cuit.basin.base.biz.entity.ModelLog;
import com.cuit.basin.base.biz.mq.redismq.producer.ModelTaskProducer;
import com.cuit.basin.framework.common.constant.ModleTaskEnum;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.kafka.core.api.KafkaProducerApi;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.UUID;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.basin.base.biz.service.model
 * @Author: HouPJ
 * @Date: 2022/12/23 9:58
 * @Description: TODO
 */
@Slf4j
@Service
public class ModelOperationTaskServiceImpl implements ModelOperationTaskService{

    @Resource
    private ModelLogMapper modelLogMapper;
    @Resource
    private KafkaProducerApi kafkaProducerServer;
    @Resource
    private ModelTaskProducer modelTaskProducer;
    @Resource
    private ModelProgressService modelProgressService;

    @Override
    public ModelTaskInfo initModelTask(InitTaskParams initTaskParams) {
        // 若模型运行方式为null，设置为手动运行
        if (ObjectUtil.isNull(initTaskParams.getRm())) {
            initTaskParams.setRm(ModleTaskEnum.TASK_METHOD_MANUAL.getTypeName());
        }
        ModelLog modelLog = ModelLogConvert.INSTANCE.convert(initTaskParams);
        // 设置模型任务默认值
        initModelTaskDo(modelLog);
        // 插入数据库
        modelLogMapper.insert(modelLog);
        // 刷新模型任务缓存
        modelTaskProducer.sendModelTaskRefreshMessage();
        initTaskParams.setId(modelLog.getId());
        // 发送模型启动消息
        try {
            kafkaProducerServer.sendMessage(KafkaTopic.TOPIC_MODEL_START, JSONUtil.toJsonStr(initTaskParams));
//            kafkaProducerServer.sendMessage(KafkaTopic.TOPIC_MODEL_START_TEST, JSONUtil.toJsonStr(initTaskParams));
            log.info("消息发送成功！" + JSONUtil.toJsonStr(initTaskParams));
        }catch (Exception errorMessage) {
            log.error("模型启动消息发送失败！");
            // 处理模型异常停止情况
            stopModelWithError(modelLog);
            // 抛出模型运行异常，全局捕获
            throw new ErrorMessage("model-exception:模型启动消息发送失败！");
        }
        // 初始化并返回 ModelTaskInfo
        return initModelTaskInfo(modelLog);
    }

    @Override
    public void stopModelWithError(String modelTaskId) {
        stopModelWithError(modelLogMapper.selectById(modelTaskId));
    }

    @Override
    public ModelTaskInfo obtainTaskProgress(String modelTaskId) {
        return ModelLogConvert.INSTANCE.convert(modelProgressService.obtainModelTaskWithId(modelTaskId));
    }

    @Override
    public void modelTaskRefreshHandler(SuccessTaskParams successTaskParams) {
        // 查询当前模型任务DO
        ModelLog modelLog = modelLogMapper.selectById(successTaskParams.getId());
        // 设置成功时默认参数
        if (ObjectUtil.isNull(modelLog)) {
            return;
        }
        initModelTaskDoWithSuccess(modelLog,successTaskParams);
        log.info("-----开始入库，当前模型日志对象为：{}",modelLog.toString());
        // 入库
        modelLogMapper.updateById(modelLog);
        // 发送更新消息
        modelTaskProducer.sendModelTaskRefreshMessage();
    }

    @Override
    public void modelTaskExceptionHandler(String taskId) {
        stopModelWithError(taskId);
    }

    @Override
    public void modelTaskSuccessHandler(SuccessTaskParams successTaskParams) {
        // 查询当前模型任务DO
        ModelLog modelLog = modelLogMapper.selectById(successTaskParams.getId());
        // 设置成功时默认参数
        if (ObjectUtil.isNull(modelLog)) {
            return;
        }
        modelLog.setIsFinish(ModleTaskEnum.TASK_FINISH.getType());
        modelLog.setEt(successTaskParams.getEt());
        modelLog.setRt(Integer.valueOf(calTimeInterval(modelLog.getSt(),successTaskParams.getEt())));
        log.info("开始入库，当前模型日志对象为：{}",modelLog.toString());
        // 入库
        modelLogMapper.updateById(modelLog);
        // 发送更新消息
        modelTaskProducer.sendModelTaskRefreshMessage();
    }

    private void initModelTaskDoWithSuccess(ModelLog modelLog, SuccessTaskParams success) {
        modelLog.setEt(success.getEt());
        modelLog.setProcess(success.getProcess());
        if (success.getProcess().equals(100.00)) {
            modelLog.setIsFinish(1);
        }else {
            modelLog.setIsFinish(success.getIsFinish());
        }
        modelLog.setRt(Integer.valueOf(calTimeInterval(modelLog.getSt(),success.getEt())));
    }

    private String calTimeInterval(LocalDateTime st, LocalDateTime et) {
        Duration duration = Duration.between(st,et);
        //相差的分钟数
        long minutes = duration.toMinutes();
        log.info("运行时间为{}",minutes);
        if (minutes == 0) {
            minutes = 1;
        }
        return String.valueOf(minutes);
    }

    /**
     * 模型发生异常时调用
     * @param modelLog
     */
    private void stopModelWithError(ModelLog modelLog) {
        modelLog.setProcess(0.0);
        modelLog.setIsFinish(ModleTaskEnum.TASK_ERROR_STOP.getType());
        modelLog.setEt(LocalDateTime.now());
        modelLogMapper.updateById(modelLog);
        // 发送刷新缓存消息
        modelTaskProducer.sendModelTaskRefreshMessage();
    }

    private ModelTaskInfo initModelTaskInfo(ModelLog modelLog) {
        return ModelLogConvert.INSTANCE.convert(modelLog);
    }

    private static void initModelTaskDo(ModelLog modelLog) {
        modelLog.setId(UUID.randomUUID().toString());
        if (ObjectUtil.isNull(modelLog.getRm())) {
            modelLog.setRm(ModleTaskEnum.TASK_METHOD_MANUAL.getTypeName());
        }
        modelLog.setIsFinish(ModleTaskEnum.TASK_RUNNING.getType());
        modelLog.setSt(LocalDateTime.now());
        modelLog.setProcess(0.0);
    }
}
