package com.cuit.basin.base.biz.service.model;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.cuit.basin.base.biz.controller.model.vo.SuccessTaskParams;
import com.cuit.basin.base.biz.convert.ModelLogConvert;
import com.cuit.basin.base.biz.dao.ModelLogMapper;
import com.cuit.basin.base.biz.entity.ModelLog;
import com.cuit.basin.base.biz.mq.model.ModelMessage;
import com.cuit.basin.base.biz.mq.redismq.producer.ModelTaskProducer;
import com.cuit.basin.base.websocket.server.WebSocketServer;
import com.cuit.basin.framework.common.utils.other.JsonUtils;
import com.google.common.collect.ImmutableMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.basin.base.biz.service.model
 * @Author: HouPJ
 * @Date: 2022/12/23 9:54
 * @Description: 模型进度管理服务
 */
@Service
@Slf4j
public class ModelProgressServiceImpl implements ModelProgressService{

    /**
     * 定时执行 {@link #schedulePeriodicRefresh()} 的周期 5分钟自动刷新一次
     * 因为已经通过 Redis Pub/Sub 机制，所以频率不需要高
     */
    private static final long SCHEDULER_PERIOD = 5 * 60 * 1000L;

    /**
     * 模型任务缓存
     * key：任务编号
     *
     * 这里声明 volatile 修饰的原因是，每次刷新时，直接修改指向
     */
    private volatile Map<String, ModelLog> taskProgressCache;


   @Resource
   private ModelLogMapper modelLogMapper;
   @Resource
   private WebSocketServer webSocketServer;
   @Resource
   private ModelTaskProducer producer;

    /**
     * 初始化 {@link #taskProgressCache}
     *
     * PostConstruct注解用于在初始化当前service自动调用此方法
     */
    @Override
    @PostConstruct
    public synchronized void initLocalCache() {
        // 初始化任务列表缓存
        List<ModelLog> modelTasks = modelLogMapper.initTaskList();
        if(ObjectUtil.isNull(modelTasks)) {
            return;
        }
        // 构建菜单和权限缓存
        ImmutableMap.Builder<String, ModelLog> taskProgressCacheBuilder = ImmutableMap.builder();
        modelTasks.forEach(task -> {
            taskProgressCacheBuilder.put(task.getId(), task);
        });
        taskProgressCache = taskProgressCacheBuilder.build();
        log.info("[initLocalCache][模型任务缓存，数量为:{}]", modelTasks.size());
        // 推送给websocket
        webSocketServer.sendGroupInfo(JSONUtil.toJsonStr(new ModelMessage("MODEL-PROGRESS","Model progress has refreshed!")));
    }

    @Override
    public void refreshToWebsocket() {
        webSocketServer.sendGroupInfo(JsonUtils.toJsonPrettyString(taskProgressCache));
    }

    @Override
    public Map<String, ModelLog> obtainModelTask() {
        return taskProgressCache;
    }

    @Override
    public void refreshTask(SuccessTaskParams successTaskParams) {
        modelLogMapper.updateById(ModelLogConvert.INSTANCE.convert(successTaskParams));
        // 发送刷新消息
        producer.sendModelTaskRefreshMessage();
        // 更新到websocket
        refreshToWebsocket();
    }

    @Override
    public ModelLog obtainModelTaskWithId(String modelTaskId) {
        // 先从缓存里取
        if (taskProgressCache.containsKey(modelTaskId)) {
            return taskProgressCache.get(modelTaskId);
        }
        return modelLogMapper.selectById(modelTaskId);
    }

    /**
     * 每隔五分钟刷新一下缓存
     */
    @Scheduled(fixedDelay = SCHEDULER_PERIOD, initialDelay = SCHEDULER_PERIOD)
    public void schedulePeriodicRefresh() {
        initLocalCache();
    }




}
