package com.cuit.basin.framework.log.aop;


import cn.hutool.json.JSONUtil;

import com.cuit.basin.framework.common.constant.ModuleEnum;
import com.cuit.basin.framework.common.pojo.HydrologyMessage;
import com.cuit.basin.framework.common.pojo.MeteorogyMessage;
import com.cuit.basin.framework.common.pojo.ModelMessage;
import com.cuit.basin.framework.common.utils.other.RedisUtil;
import com.cuit.basin.framework.common.utils.other.RequestUtil;
import com.cuit.basin.framework.kafka.core.api.KafkaProducerApi;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import com.cuit.basin.framework.log.annotation.AopLog;
import com.cuit.basin.framework.log.dto.SysLogDto;
import com.cuit.basin.framework.log.server.OperationFrameworkService;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.CodeSignature;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.sriver.sriversys.comon.aop
 * @Author: HouPJ
 * @Date: 2022/11/12 17:25
 * @Description: aop通用日志记录切面
 */

@Aspect
@Component
public class AopLogAspect {

    @Resource
    private KafkaProducerApi kafkaProducer;
    @Resource
    private OperationFrameworkService operationFrameworkService;

    private final RedisUtil redisUtil = new RedisUtil();


    /**
     * 请求地址
     */
    private String requestPath = null;
    /**
     * 访问时间
     */
    private LocalDateTime startTimeMillis = null;
    /**
     * 结束时间
     */
    private long endTimeMillis = 0;
    /**
     * 操作人
     */
    private String account = null;
    /**
     * 请求
     */
    private HttpServletRequest request = null;
    /**
     * 操作类型
     */
    private Integer sysType = null;


    /**
     * 注解的位置
     */
    @Pointcut("@annotation(com.cuit.basin.framework.log.annotation.AopLog)")
    public void logPointCut() {
    }

    /**
     * @param joinPoint
     * @Description 前置通知  方法调用前触发   记录开始时间,从请求中获取用户名
     */
    @Before(value = "logPointCut()")
    public void before(JoinPoint joinPoint) {
        //1.获取接口访问时间
        startTimeMillis = LocalDateTime.now();
        //2.获取当前用户名
        account = (String) redisUtil.get(RequestUtil.releaseTokenFromRequest(getHttpServletRequest()));
    }

    /**
     * @param joinPoint
     * @return
     * @Description 获取入参方法参数
     */
    public Map<String, Object> getNameAndValue(JoinPoint joinPoint) {
        Map<String, Object> param = new HashMap<>();
        Object[] paramValues = joinPoint.getArgs();
        String[] paramNames = ((CodeSignature) joinPoint.getSignature()).getParameterNames();
        for (int i = 0; i < paramNames.length; i++) {
            if (paramValues[i] instanceof Integer || paramValues[i] instanceof String) {
                param.put(paramNames[i], paramValues[i]);
            }
        }
        return param;
    }


    /**
     * @param joinPoint
     * @Description 后置通知    方法调用后触发   记录结束时间 ,操作人 ,入参等
     */
    @After(value = "logPointCut()")
    public void after(JoinPoint joinPoint) {
        //3.判断是否需要进行操作描述日志记录
        request = getHttpServletRequest();
        // 获取自定义注解
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        AopLog aopLog = method.getAnnotation(AopLog.class);
        sendMessage(aopLog.module());
        // 构建日志对象并异步入库
        operationFrameworkService.createOperateLog(SysLogDto.builder().isValid(1)
                .module(aopLog.module())
                .opration(aopLog.operation())
                .sysType(aopLog.sysType())
                .visitTime(startTimeMillis)
                .opType(aopLog.opType())
                .method(joinPoint.getSignature().getName())
                .ip(getIpAddress())
                .executionTime(Duration.between(startTimeMillis, LocalDateTime.now()).toMillis() + "毫秒")
                .userName(account)
                .build());
    }

    private void sendMessage(String module) {
        if (Integer.valueOf(module).equals(ModuleEnum.MODEL_MANAGE.getType())) {
            kafkaProducer.sendMessage(KafkaTopic.TOPIC_MODEL_LOG, JSONUtil.toJsonStr(new ModelMessage("REFRESH","A new log insert!")));
        }else if (Integer.valueOf(module).equals(ModuleEnum.HYDROLOGY.getType())) {
            kafkaProducer.sendMessage(KafkaTopic.TOPIC_DATA_ETL_HYDRO, JSONUtil.toJsonStr(new HydrologyMessage("REFRESH","A new log insert!")));
        }else if (Integer.valueOf(module).equals(ModuleEnum.METEOROLOGY.getType())) {
            kafkaProducer.sendMessage(KafkaTopic.TOPIC_DATA_ETL_METEOROLOGY,JSONUtil.toJsonStr(new MeteorogyMessage("REFRESH","A new log insert!")));
        }
    }

    /**
     * 获取ip地址
     *
     * @return
     */
    public String getIpAddress() {
        HttpServletRequest request = getHttpServletRequest();
        String ip = request.getHeader("x-forwarded-for");
        if (ip != null && ip.length() != 0 && !"unknown".equalsIgnoreCase(ip)) {
            // 多次反向代理后会有多个ip值，第一个ip才是真实ip
            if (ip.indexOf(",") != -1) {
                ip = ip.split(",")[0];
            }
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Real-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip.equals("0:0:0:0:0:0:0:1") ? "127.0.0.1" : ip;
    }

    /**
     * @Description: 获取request
     */
    public HttpServletRequest getHttpServletRequest() {
        RequestAttributes ra = RequestContextHolder.getRequestAttributes();
        ServletRequestAttributes sra = (ServletRequestAttributes) ra;
        HttpServletRequest request = sra.getRequest();
        return request;
    }


    /**
     * @param joinPoint
     * @return 环绕通知
     * @throws Throwable
     */
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
        return null;
    }

    /**
     * @param joinPoint
     * @Description 异常通知
     */
    public void throwing(JoinPoint joinPoint) {
        System.out.println("异常通知");
    }


}