package com.cuit.basin.base.redismq.core.pubsub;

import cn.hutool.core.util.TypeUtil;
import com.cuit.basin.base.redismq.core.RedisMQTemplate;
import com.cuit.basin.base.redismq.core.message.AbstractRedisMessage;
import com.cuit.basin.framework.common.utils.other.JsonUtils;
import lombok.Setter;
import lombok.SneakyThrows;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;

import java.lang.reflect.Type;


/**
 * @BelongsProject: steward
 * @BelongsPackage: com.yhxc.server.framework.redismq.core.pubsub
 * @Author: HouPJ
 * @Date: 2022/12/21 10:39
 * @Description: Redis Pub/Sub 监听器抽象类，用于实现广播消费
 */
public abstract class AbstractChannelMessageListener<T extends AbstractChannelMessage> implements MessageListener {

    /**
     * 消息类型 即 T 的具体实现类名（如）
     */
    private final Class<T> messageType;
    /**
     * Redis Channel 即具体 redis 消息的唯一主题
     */
    private final String channel;
    /**
     * RedisMQTemplate
     */
    @Setter
    private RedisMQTemplate redisMQTemplate;

    @SneakyThrows
    protected AbstractChannelMessageListener() {
        this.messageType = getMessageClass();
        this.channel = messageType.newInstance().getChannel();
    }

    /**
     * 获得 Sub 订阅的 Redis Channel 通道
     *
     * @return channel
     */
    public final String getChannel() {
        return channel;
    }

    @Override
    public final void onMessage(Message message, byte[] bytes) {
        T messageObj = JsonUtils.parseObject(message.getBody(), messageType);
        try {
            // 可添加拦截器，实现消息的前处理

            // 消费消息
            this.onMessage(messageObj);
        } finally {
            // 可添加拦截器，实现消息的后处理
        }
    }

    /**
     * 处理消息
     *
     * @param message 消息
     */
    public abstract void onMessage(T message);

    /**
     * 通过解析类上的泛型，获得消息类型
     *
     * @return 消息类型
     */
    @SuppressWarnings("unchecked")
    private Class<T> getMessageClass() {
        Type type = TypeUtil.getTypeArgument(getClass(), 0);
        if (type == null) {
            throw new IllegalStateException(String.format("类型(%s) 需要设置消息类型", getClass().getName()));
        }
        return (Class<T>) type;
    }

    /**
     * 利用拦截器，实现消费前处理
     * @param message 消息
     */
    private void consumeMessageBefore(AbstractRedisMessage message) {

    }

    /**
     * 利用拦截器，实现消费后处理
     * @param message 消息
     */
    private void consumeMessageAfter(AbstractRedisMessage message) {

    }

}

