package com.cuit.basin.model.mq;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import com.cuit.basin.model.handler.GlobalExceptionHandler;
import com.cuit.basin.model.parse.ModelTaskManager;
import com.cuit.basin.model.pojo.InitTaskParams;
import com.cuit.basin.model.service.ModelTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.sriver.sriversys.mq.model
 * @Author: HouPJ
 * @Date: 2022/12/22 16:27
 * @Description: 模型相关消息消费者
 */
@Component
@Slf4j
public class ModelMQConsummer {

    @Resource
    private ModelTaskService modelTaskService;
    @Resource
    private ModelTaskManager modelTaskManager;
    @Resource
    private GlobalExceptionHandler exceptionHandler;


    /**
     * 监听模型开始运行消息
     *
     * @param message message
     * @param ack     偏移
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_START}, groupId = "model-task-start", concurrency = "3", errorHandler = "consumerAwareErrorHandler")
    public void listenModelStart(String message, Acknowledgment ack) {
        log.info("[Consumer-listenModelStart]:[线程：{}，接收到主题为{}的消息，content:{}]", Thread.currentThread().getId(), KafkaTopic.TOPIC_MODEL_START, message);
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        //-----------------------模型业务逻辑处理-----------------------//
        // 转换message中的模型任务初始化参数
        InitTaskParams initTaskParams = null;
        try {
            initTaskParams = JSONUtil.toBean(message, InitTaskParams.class);
            // 异步调用模型服务(测试用例)
            modelTaskManager.runModel(initTaskParams);
        }catch (Exception e) {
            // 发送模型运行异常消息
            assert initTaskParams != null;
            // 记录异常日志
            exceptionHandler.createExceptionLog(e);
            modelTaskService.sendModelTaskException(initTaskParams.getId());
        }

        // 手动提交偏移量
        try {
            ack.acknowledge();
            log.info("[Consumer-listenModelStart]:消息：{}消费完成...", message);
        } catch (Exception e) {
            log.error("[Consumer-listenModelStart]:消费模型启动消息异常，errMsg:{}", e.getMessage(), e);
            // 抛出异常，全局捕获
            throw e;
        }
    }

    /**
     * 监听模型停止运行消息
     *
     * @param message message
     * @param ack     偏移
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_MODEL_STOP}, groupId = "model-task-stop", concurrency = "3",errorHandler = "consumerAwareErrorHandler")
    public void listenModelStop(String message, Acknowledgment ack) {
        log.info("[Consumer-listenModelStop]:线程：{}，接收到主题为{}的消息，content:{}", Thread.currentThread().getId(), KafkaTopic.TOPIC_MODEL_STOP, message);
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        //-----------------------模型业务逻辑处理-----------------------//
        // 异步调用模型服务(测试用例)
        modelTaskManager.stopModelTask(message);
        // 手动提交偏移量
        try {
            ack.acknowledge();
            log.info("[Consumer-listenModelStop]:消息：{}消费完成...", message);
        } catch (Exception e) {
            log.error("[Consumer-listenModelStop]:消费模型启动消息异常，errMsg:{}", e.getMessage(), e);
            // 抛出异常，全局捕获
            throw e;
        }
    }


}
